""" Access control testing
"""

from django.test import override_settings

from core_main_app.access_control.exceptions import AccessControlError
from core_main_app.components.template_version_manager import (
    api as template_vm_api,
)
from core_main_app.utils.integration_tests.integration_base_test_case import (
    IntegrationBaseTestCase,
)
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import create_mock_request
from core_main_app.components.template_version_manager import access_control

from tests.components.template_version_manager.fixtures.fixtures import (
    TemplateVersionManagerAccessControlFixtures,
    TemplateVersionManagerOrderingFixtures,
)


fixture_template_vm = TemplateVersionManagerAccessControlFixtures()
fixture_template_vm_ordering = TemplateVersionManagerOrderingFixtures()


class TestTemplateVersionManagerGet(IntegrationBaseTestCase):
    """TestTemplateVersionManagerGet"""

    fixture = fixture_template_vm

    def setUp(self):
        """setUp

        Returns:

        """
        self.anonymous_user = create_mock_user(user_id=None, is_anonymous=True)
        self.user1 = create_mock_user(user_id="1")
        self.staff_user1 = create_mock_user(user_id="1", is_staff=True)
        self.superuser1 = create_mock_user(user_id="1", is_superuser=True)
        self.fixture.insert_data()

    def test_get_user_version_manager_as_anonymous_raises_access_control_error(
        self,
    ):
        """test get user version manager as anonymous raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            template_vm_api.get_by_id(
                self.fixture.user1_tvm.id, request=mock_request
            )

    @override_settings(CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT=True)
    def test_get_user_version_manager_as_anonymous_with_access_right_raises_access_control_error(
        self,
    ):
        """test get user version manager as anonymous with access right raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            template_vm_api.get_by_id(
                self.fixture.user1_tvm.id, request=mock_request
            )

    def test_get_global_version_manager_as_anonymous_raises_access_control_error(
        self,
    ):
        """test get global version manager as anonymous raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            template_vm_api.get_by_id(
                self.fixture.global_tvm.id, request=mock_request
            )

    @override_settings(CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT=True)
    def test_get_global_version_manager_as_anonymous_with_access_right_returns_global_template(
        self,
    ):
        """test get global version manager as anonymous with access right returns version manager

        Returns:

        """
        mock_request = create_mock_request(user=self.anonymous_user)
        version_manager = template_vm_api.get_by_id(
            self.fixture.global_tvm.id, request=mock_request
        )
        self.assertEqual(version_manager, self.fixture.global_tvm)

    def test_get_own_version_manager_as_user_returns_version_manager(self):
        """test get own version manager as user returns version manager

        Returns:

        """
        mock_request = create_mock_request(user=self.user1)
        version_manager = template_vm_api.get_by_id(
            self.fixture.user1_tvm.id, request=mock_request
        )
        self.assertEqual(version_manager, self.fixture.user1_tvm)

    def test_global_version_manager_as_user_returns_version_manager(self):
        """test global version manager as user returns version manager

        Returns:

        """
        mock_request = create_mock_request(user=self.user1)
        version_manager = template_vm_api.get_by_id(
            self.fixture.global_tvm.id, request=mock_request
        )
        self.assertEqual(version_manager, self.fixture.global_tvm)

    def test_get_other_users_version_manager_raises_access_control_error(self):
        """test get other users version manager raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.user1)
        with self.assertRaises(AccessControlError):
            template_vm_api.get_by_id(
                self.fixture.user2_tvm.id, request=mock_request
            )

    def test_get_any_version_manager_as_superuser_returns_version_manager(
        self,
    ):
        """test get any version manager as superuser returns version manager

        Returns:

        """
        mock_request = create_mock_request(user=self.superuser1)
        version_manager = template_vm_api.get_by_id(
            self.fixture.user1_tvm.id, request=mock_request
        )
        self.assertEqual(version_manager, self.fixture.user1_tvm)
        version_manager = template_vm_api.get_by_id(
            self.fixture.user2_tvm.id, request=mock_request
        )
        self.assertEqual(version_manager, self.fixture.user2_tvm)
        version_manager = template_vm_api.get_by_id(
            self.fixture.global_tvm.id, request=mock_request
        )
        self.assertEqual(version_manager, self.fixture.global_tvm)

    def test_get_other_users_version_manager_as_staff_raises_access_control_error(
        self,
    ):
        """test get other users version manager as staff raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.staff_user1)
        with self.assertRaises(AccessControlError):
            template_vm_api.get_by_id(
                self.fixture.user2_tvm.id, request=mock_request
            )


class TestTemplateVersionManagerGetByIdList(IntegrationBaseTestCase):
    """TesTemplateVersionManagerGetByIdList"""

    fixture = fixture_template_vm

    def setUp(self):
        """setUp

        Returns:

        """
        self.anonymous_user = create_mock_user(user_id=None, is_anonymous=True)
        self.user1 = create_mock_user(user_id="1")
        self.staff_user1 = create_mock_user(user_id="1", is_staff=True)
        self.superuser1 = create_mock_user(user_id="1", is_superuser=True)
        self.fixture.insert_data()

    def test_get_by_id_list_user_version_manager_as_anonymous_raises_access_control_error(
        self,
    ):
        """test get by id list user version manager as anonymous raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            template_vm_api.get_by_id_list(
                [str(self.fixture.user1_tvm.id)], request=mock_request
            )

    @override_settings(CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT=True)
    def test_get_by_id_list_user_version_manager_as_anonymous_with_access_right_raises_access_control_error(
        self,
    ):
        """test get by id list user version manager as anonymous with access right raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            template_vm_api.get_by_id_list(
                [str(self.fixture.user1_tvm.id)], request=mock_request
            )

    def test_get_by_id_list_global_version_manager_as_anonymous_raises_access_control_error(
        self,
    ):
        """test get by id list global version manager as anonymous raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            template_vm_api.get_by_id_list(
                [str(self.fixture.global_tvm.id)], request=mock_request
            )

    @override_settings(CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT=True)
    def test_get_by_id_list_global_version_manager_as_anonymous_with_access_right_returns_version_manager(
        self,
    ):
        """test get by id list global version manager as anonymous with access right version manager

        Returns:

        """
        mock_request = create_mock_request(user=self.anonymous_user)
        version_managers = template_vm_api.get_by_id_list(
            [str(self.fixture.global_tvm.id)], request=mock_request
        )
        self.assertTrue(self.fixture.global_tvm in list(version_managers))

    def test_get_by_id_list_user_version_manager_as_user_returns_version_manager(
        self,
    ):
        """test get by id list user version manager as user returns template version manager

        Returns:

        """
        mock_request = create_mock_request(user=self.user1)
        version_managers = template_vm_api.get_by_id_list(
            [str(self.fixture.user1_tvm.id)], request=mock_request
        )
        self.assertTrue(self.fixture.user1_tvm in list(version_managers))
        self.assertTrue(self.fixture.user2_tvm not in list(version_managers))
        self.assertTrue(self.fixture.global_tvm not in list(version_managers))

    def test_get_by_id_list_global_version_manager_as_user_returns_version_manager(
        self,
    ):
        """test get by id list global version manager as user returns version manager

        Returns:

        """
        mock_request = create_mock_request(user=self.user1)
        version_managers = template_vm_api.get_by_id_list(
            [str(self.fixture.global_tvm.id)], request=mock_request
        )
        self.assertTrue(self.fixture.user1_tvm not in list(version_managers))
        self.assertTrue(self.fixture.user2_tvm not in list(version_managers))
        self.assertTrue(self.fixture.global_tvm in list(version_managers))

    def test_get_by_id_list_other_user_version_manager_as_user_raises_access_control_error(
        self,
    ):
        """test get by id list other user version manager as user raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.user1)
        with self.assertRaises(AccessControlError):
            template_vm_api.get_by_id_list(
                [str(self.fixture.user2_tvm.id)], request=mock_request
            )

    def test_get_by_id_list_user_version_manager_as_staff_returns_version_manager(
        self,
    ):
        """test get by id list user version manager as staff returns version manager

        Returns:

        """
        mock_request = create_mock_request(user=self.staff_user1)
        version_managers = template_vm_api.get_by_id_list(
            [str(self.fixture.user1_tvm.id)], request=mock_request
        )
        self.assertTrue(self.fixture.user1_tvm in list(version_managers))
        self.assertTrue(self.fixture.user2_tvm not in list(version_managers))
        self.assertTrue(self.fixture.global_tvm not in list(version_managers))

    def test_get_by_id_list_global_version_manager_as_staff_returns_version_manager(
        self,
    ):
        """test get by id list global version manager as staff returns version manager

        Returns:

        """
        mock_request = create_mock_request(user=self.staff_user1)
        version_managers = template_vm_api.get_by_id_list(
            [str(self.fixture.global_tvm.id)], request=mock_request
        )
        self.assertTrue(self.fixture.user1_tvm not in list(version_managers))
        self.assertTrue(self.fixture.user2_tvm not in list(version_managers))
        self.assertTrue(self.fixture.global_tvm in list(version_managers))

    def test_get_by_id_list_other_user_version_manager_as_staff_raises_access_control_error(
        self,
    ):
        """test get by id list other user version manager as staff raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.staff_user1)
        with self.assertRaises(AccessControlError):
            template_vm_api.get_by_id_list(
                [str(self.fixture.user2_tvm.id)], request=mock_request
            )

    def test_get_by_id_list_user_version_manager_as_superuser_returns_version_manager(
        self,
    ):
        """test get by id list user version manager as superuser returns version manager

        Returns:

        """
        mock_request = create_mock_request(user=self.superuser1)
        version_managers = template_vm_api.get_by_id_list(
            [str(self.fixture.user1_tvm.id)], request=mock_request
        )
        self.assertTrue(self.fixture.user1_tvm in list(version_managers))
        self.assertTrue(self.fixture.user2_tvm not in list(version_managers))
        self.assertTrue(self.fixture.global_tvm not in list(version_managers))

    def test_get_by_id_list_global_version_manager_as_superuser_returns_version_manager(
        self,
    ):
        """test get by id list global version manager as superuser returns version manager

        Returns:

        """
        mock_request = create_mock_request(user=self.superuser1)
        version_managers = template_vm_api.get_by_id_list(
            [str(self.fixture.global_tvm.id)], request=mock_request
        )
        self.assertTrue(self.fixture.user1_tvm not in list(version_managers))
        self.assertTrue(self.fixture.user2_tvm not in list(version_managers))
        self.assertTrue(self.fixture.global_tvm in list(version_managers))

    def test_get_by_id_list_other_user_version_manager_as_superuser_returns_version_manager(
        self,
    ):
        """test get by id list other user version manager as superuser returns version manager

        Returns:

        """
        mock_request = create_mock_request(user=self.superuser1)
        version_managers = template_vm_api.get_by_id_list(
            [str(self.fixture.user2_tvm.id)], request=mock_request
        )
        self.assertTrue(self.fixture.user1_tvm not in list(version_managers))
        self.assertTrue(self.fixture.user2_tvm in list(version_managers))
        self.assertTrue(self.fixture.global_tvm not in list(version_managers))


class TestTemplateVersionManagerGetActiveGlobalVersionManagerByTitle(
    IntegrationBaseTestCase
):
    """TestTemplateVersionManagerGetActiveGlobalVersionManagerByTitle"""

    fixture = fixture_template_vm

    def setUp(self):
        """setUp

        Returns:

        """
        self.anonymous_user = create_mock_user(user_id=None, is_anonymous=True)
        self.user1 = create_mock_user(user_id="1")
        self.staff_user1 = create_mock_user(user_id="1", is_staff=True)
        self.superuser1 = create_mock_user(user_id="1", is_superuser=True)
        self.fixture.insert_data()

    def test_get_active_global_version_manager_by_title_as_anonymous_raises_access_control_error(
        self,
    ):
        """test get active global version manager by title as anonymous raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            template_vm_api.get_active_global_version_manager_by_title(
                self.fixture.global_tvm.title, request=mock_request
            )

    @override_settings(CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT=True)
    def test_get_active_global_version_manager_by_title_as_anonymous_with_access_right_returns_version_manager(
        self,
    ):
        """test get active global version manager by title as anonymous with access right returns version manager

        Returns:

        """
        mock_request = create_mock_request(user=self.anonymous_user)
        version_manager = (
            template_vm_api.get_active_global_version_manager_by_title(
                self.fixture.global_tvm.title, request=mock_request
            )
        )
        self.assertEqual(version_manager, self.fixture.global_tvm)

    def test_get_active_global_version_manager_as_user_returns_version_manager(
        self,
    ):
        """test get active global version manager as user returns version manager

        Returns:

        """
        mock_request = create_mock_request(user=self.user1)
        version_manager = (
            template_vm_api.get_active_global_version_manager_by_title(
                self.fixture.global_tvm.title, request=mock_request
            )
        )
        self.assertEqual(version_manager, self.fixture.global_tvm)

    def test_get_active_global_version_manager_by_title_as_superuser_returns_version_manager(
        self,
    ):
        """test get active global version manager by title as superuser returns version manager

        Returns:

        """
        mock_request = create_mock_request(user=self.superuser1)
        version_manager = (
            template_vm_api.get_active_global_version_manager_by_title(
                self.fixture.global_tvm.title, request=mock_request
            )
        )
        self.assertEqual(version_manager, self.fixture.global_tvm)

    def test_get_active_global_version_manager_by_title_as_staff_raises_access_control_error(
        self,
    ):
        """test get active global version manager by title as staff raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.staff_user1)
        version_manager = (
            template_vm_api.get_active_global_version_manager_by_title(
                self.fixture.global_tvm.title, request=mock_request
            )
        )
        self.assertEqual(version_manager, self.fixture.global_tvm)


class TestTemplateVersionManagerInsert(IntegrationBaseTestCase):
    """TestTemplateVersionManagerInsert"""

    fixture = fixture_template_vm

    def setUp(self):
        """setUp

        Returns:

        """
        self.anonymous_user = create_mock_user(user_id=None, is_anonymous=True)
        self.user1 = create_mock_user(user_id="1")
        self.staff_user1 = create_mock_user(user_id="1", is_staff=True)
        self.superuser1 = create_mock_user(user_id="1", is_superuser=True)
        self.fixture.insert_data()

    def test_insert_user_template_as_anonymous_raises_access_control_error(
        self,
    ):
        """test insert user template as anonymous raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            template_vm_api.insert(
                self.fixture.user1_tvm,
                self.fixture.user1_template,
                request=mock_request,
            )

    @override_settings(CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT=True)
    def test_insert_user_template_as_anonymous_with_access_right_raises_access_control_error(
        self,
    ):
        """test insert user template as anonymous with access right raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            template_vm_api.insert(
                self.fixture.user1_tvm,
                self.fixture.user1_template,
                request=mock_request,
            )

    def test_insert_global_template_as_anonymous_raises_access_control_error(
        self,
    ):
        """test insert global template as anonymous raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            template_vm_api.insert(
                self.fixture.global_tvm,
                self.fixture.global_template,
                request=mock_request,
            )

    @override_settings(CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT=True)
    def test_insert_global_template_as_anonymous_with_access_right_raises_access_control_error(
        self,
    ):
        """test insert global template as anonymous with access right raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            template_vm_api.insert(
                self.fixture.global_tvm,
                self.fixture.global_template,
                request=mock_request,
            )

    def test_insert_own_template_as_user_saves(self):
        """test insert own template as user saves

        Returns:

        """
        mock_request = create_mock_request(user=self.user1)
        template_vm_api.insert(
            self.fixture.user1_tvm,
            self.fixture.user1_template,
            request=mock_request,
        )

    def test_insert_other_users_template_as_user_raises_access_control_error(
        self,
    ):
        """test insert other users template as user raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.user1)
        with self.assertRaises(AccessControlError):
            template_vm_api.insert(
                self.fixture.user2_tvm,
                self.fixture.user2_template,
                request=mock_request,
            )

    def test_insert_global_template_as_user_raises_access_control_error(self):
        """test insert global template as user raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.user1)
        with self.assertRaises(AccessControlError):
            template_vm_api.insert(
                self.fixture.global_tvm,
                self.fixture.global_template,
                request=mock_request,
            )

    def test_insert_own_template_as_staff_saves(self):
        """test insert own template as staff saves

        Returns:

        """
        mock_request = create_mock_request(user=self.staff_user1)
        template_vm_api.insert(
            self.fixture.user1_tvm,
            self.fixture.user1_template,
            request=mock_request,
        )

    def test_insert_other_users_template_as_staff_raises_access_control_error(
        self,
    ):
        """test insert other users template as staff raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.staff_user1)
        with self.assertRaises(AccessControlError):
            template_vm_api.insert(
                self.fixture.user2_tvm,
                self.fixture.user2_template,
                request=mock_request,
            )

    def test_insert_global_template_as_staff_saves(self):
        """test insert global template as staff saves

        Returns:

        """
        mock_request = create_mock_request(user=self.staff_user1)
        template_vm_api.insert(
            self.fixture.global_tvm,
            self.fixture.global_template,
            request=mock_request,
        )

    def test_insert_own_template_as_superuser_saves(self):
        """test insert own template as superuser saves

        Returns:

        """
        mock_request = create_mock_request(user=self.superuser1)
        template_vm_api.insert(
            self.fixture.user1_tvm,
            self.fixture.user1_template,
            request=mock_request,
        )

    def test_insert_other_users_template_as_superuser_saves(self):
        """test insert other users template as superuser saves

        Returns:

        """
        mock_request = create_mock_request(user=self.superuser1)
        template_vm_api.insert(
            self.fixture.user2_tvm,
            self.fixture.user2_template,
            request=mock_request,
        )

    def test_insert_global_template_as_superuser_saves(self):
        """test insert global template as superuser saves

        Returns:

        """
        mock_request = create_mock_request(user=self.superuser1)
        template_vm_api.insert(
            self.fixture.global_tvm,
            self.fixture.global_template,
            request=mock_request,
        )


class TestTemplateEditTitle(IntegrationBaseTestCase):
    """TestTemplateEditTitle"""

    fixture = fixture_template_vm

    def setUp(self):
        """setUp

        Returns:

        """
        self.anonymous_user = create_mock_user(user_id=None, is_anonymous=True)
        self.user1 = create_mock_user(user_id="1")
        self.staff_user1 = create_mock_user(user_id="1", is_staff=True)
        self.superuser1 = create_mock_user(user_id="1", is_superuser=True)
        self.fixture.insert_data()

    def test_edit_title_user_template_as_anonymous_raises_access_control_error(
        self,
    ):
        """test edit title user template as anonymous raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            template_vm_api.edit_title(
                self.fixture.user1_tvm, "new_name", request=mock_request
            )

    @override_settings(CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT=True)
    def test_edit_title_user_template_as_anonymous_with_access_right_raises_access_control_error(
        self,
    ):
        """test edit title user template as anonymous with access right  raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            template_vm_api.edit_title(
                self.fixture.user1_tvm, "new_name", request=mock_request
            )

    def test_edit_title_global_template_as_anonymous_raises_access_control_error(
        self,
    ):
        """test edit title global template as anonymous raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            template_vm_api.edit_title(
                self.fixture.global_tvm, "new_name", request=mock_request
            )

    @override_settings(CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT=True)
    def test_edit_title_global_template_as_anonymous_with_access_right_raises_access_control_error(
        self,
    ):
        """test edit title global template as anonymous with access right  raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            template_vm_api.edit_title(
                self.fixture.global_tvm, "new_name", request=mock_request
            )

    def test_edit_title_own_template_as_user_saves(self):
        """test edit title own template as user saves

        Returns:

        """
        mock_request = create_mock_request(user=self.user1)
        template_vm_api.edit_title(
            self.fixture.user1_tvm, "new_name", request=mock_request
        )

    def test_edit_title_other_users_template_as_user_raises_access_control_error(
        self,
    ):
        """test edit title other users template as user raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.user1)
        with self.assertRaises(AccessControlError):
            template_vm_api.edit_title(
                self.fixture.user2_tvm, "new_name", request=mock_request
            )

    def test_edit_title_global_template_as_user_raises_access_control_error(
        self,
    ):
        """test edit title global template as user raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.user1)
        with self.assertRaises(AccessControlError):
            template_vm_api.edit_title(
                self.fixture.global_tvm, "new_name", request=mock_request
            )

    def test_edit_title_own_template_as_staff_saves(self):
        """test edit title own template as staff saves

        Returns:

        """
        mock_request = create_mock_request(user=self.staff_user1)
        template_vm_api.edit_title(
            self.fixture.user1_tvm, "new_name", request=mock_request
        )

    def test_edit_title_other_users_template_as_staff_raises_access_control_error(
        self,
    ):
        """test edit title other users template as staff raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.staff_user1)
        with self.assertRaises(AccessControlError):
            template_vm_api.edit_title(
                self.fixture.user2_tvm, "new_name", request=mock_request
            )

    def test_edit_title_global_template_as_staff_saves(self):
        """test edit title global template as staff saves

        Returns:

        """
        mock_request = create_mock_request(user=self.staff_user1)
        template_vm_api.edit_title(
            self.fixture.global_tvm, "new_name", request=mock_request
        )

    def test_edit_title_own_template_as_superuser_saves(self):
        """test edit title own template as superuser saves

        Returns:

        """
        mock_request = create_mock_request(user=self.superuser1)
        template_vm_api.edit_title(
            self.fixture.user1_tvm, "new_name", request=mock_request
        )

    def test_edit_title_other_users_template_as_superuser_saves(self):
        """test edit title other users template as superuser saves

        Returns:

        """
        mock_request = create_mock_request(user=self.superuser1)
        template_vm_api.edit_title(
            self.fixture.user2_tvm, "new_name", request=mock_request
        )

    def test_edit_title_global_template_as_superuser_saves(self):
        """test edit title global template as superuser saves

        Returns:

        """
        mock_request = create_mock_request(user=self.superuser1)
        template_vm_api.edit_title(
            self.fixture.global_tvm, "new_name", request=mock_request
        )


class TestTemplateGetGlobalVersionManagers(IntegrationBaseTestCase):
    """TestTemplateGetGlobalVersionManagers"""

    fixture = fixture_template_vm

    def setUp(self):
        """setUp

        Returns:

        """
        self.anonymous_user = create_mock_user(user_id=None, is_anonymous=True)
        self.user1 = create_mock_user(user_id="1")
        self.staff_user1 = create_mock_user(user_id="1", is_staff=True)
        self.superuser1 = create_mock_user(user_id="1", is_superuser=True)
        self.fixture.insert_data()

    def test_get_global_version_managers_as_anonymous_raises_access_control_error(
        self,
    ):
        """test get global version managers as anonymous raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            template_vm_api.get_global_version_managers(request=mock_request)

    @override_settings(CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT=True)
    def test_get_global_version_managers_as_anonymous_with_access_right_returns_global_tvm(
        self,
    ):
        """test get global version managers as anonymous with access right returns global tvm

        Returns:

        """
        mock_request = create_mock_request(user=self.anonymous_user)
        list_tvm = template_vm_api.get_global_version_managers(
            request=mock_request
        )
        for tvm in list_tvm:
            self.assertEqual(tvm.user, None)

    def test_get_global_version_managers_as_user_returns_global_tvm(self):
        """test get global version managers as user returns global tvm

        Returns:

        """
        mock_request = create_mock_request(user=self.user1)
        list_tvm = template_vm_api.get_global_version_managers(
            request=mock_request
        )
        for tvm in list_tvm:
            self.assertEqual(tvm.user, None)

    def test_get_global_version_managers_as_staff_returns_global_tvm(self):
        """test get global version managers as staff returns global tvm

        Returns:

        """
        mock_request = create_mock_request(user=self.staff_user1)
        list_tvm = template_vm_api.get_global_version_managers(
            request=mock_request
        )
        for tvm in list_tvm:
            self.assertEqual(tvm.user, None)

    def test_get_global_version_managers_as_superuser_returns_global_tvm(self):
        """test get global version managers as superuser returns global tvm

        Returns:

        """
        mock_request = create_mock_request(user=self.superuser1)
        list_tvm = template_vm_api.get_global_version_managers(
            request=mock_request
        )
        for tvm in list_tvm:
            self.assertEqual(tvm.user, None)


class TestTemplateGetActiveGlobalVersionManager(IntegrationBaseTestCase):
    """TestTemplateGetActiveGlobalVersionManager"""

    fixture = fixture_template_vm

    def setUp(self):
        """setUp

        Returns:

        """
        self.anonymous_user = create_mock_user(user_id=None, is_anonymous=True)
        self.user1 = create_mock_user(user_id="1")
        self.staff_user1 = create_mock_user(user_id="1", is_staff=True)
        self.superuser1 = create_mock_user(user_id="1", is_superuser=True)
        self.fixture.insert_data()

    def test_get_active_global_version_manager_as_anonymous_raises_access_control_error(
        self,
    ):
        """test get active global version manager as anonymous raises access control error

        Returns:

        """
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            template_vm_api.get_active_global_version_manager(
                request=mock_request
            )

    @override_settings(CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT=True)
    def test_get_active_global_version_manager_as_anonymous_with_access_right_returns_global_tvm(
        self,
    ):
        """test get active global version manager as anonymous with access right returns global tvm

        Returns:

        """
        mock_request = create_mock_request(user=self.anonymous_user)
        list_tvm = template_vm_api.get_active_global_version_manager(
            request=mock_request
        )
        for tvm in list_tvm:
            self.assertEqual(tvm.user, None)

    def test_get_active_global_version_manager_as_user_returns_global_tvm(
        self,
    ):
        """test get active global version manager as user returns global tvm

        Returns:

        """
        mock_request = create_mock_request(user=self.user1)
        list_tvm = template_vm_api.get_active_global_version_manager(
            request=mock_request
        )
        for tvm in list_tvm:
            self.assertEqual(tvm.user, None)

    def test_get_active_global_version_manager_as_staff_returns_global_tvm(
        self,
    ):
        """test get active global version manager as staff returns global tvm

        Returns:

        """
        mock_request = create_mock_request(user=self.staff_user1)
        list_tvm = template_vm_api.get_active_global_version_manager(
            request=mock_request
        )
        for tvm in list_tvm:
            self.assertEqual(tvm.user, None)

    def test_get_active_global_version_manager_as_superuser_returns_global_tvm(
        self,
    ):
        """test get active global version manager as superuser returns global tvm

        Returns:

        """
        mock_request = create_mock_request(user=self.superuser1)
        list_tvm = template_vm_api.get_active_global_version_manager(
            request=mock_request
        )
        for tvm in list_tvm:
            self.assertEqual(tvm.user, None)


class TestTemplateGetActiveVersionManagerByUserId(IntegrationBaseTestCase):
    """TestTemplateGetActiveVersionManagerByUserId"""

    fixture = fixture_template_vm

    def setUp(self):
        """setUp

        Returns:

        """
        self.anonymous_user = create_mock_user(user_id=None, is_anonymous=True)
        self.user1 = create_mock_user(user_id="1")
        self.staff_user1 = create_mock_user(user_id="1", is_staff=True)
        self.superuser1 = create_mock_user(user_id="1", is_superuser=True)
        self.fixture.insert_data()

    def test_get_active_version_manager_by_user_id_as_anonymous_returns_nothing(
        self,
    ):
        """test get active version manager by user id as anonymous returns nothing

        Returns:

        """
        mock_request = create_mock_request(user=self.anonymous_user)
        list_tvm = template_vm_api.get_active_version_manager_by_user_id(
            request=mock_request
        )
        self.assertEqual(list_tvm.count(), 0)

    @override_settings(CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT=True)
    def test_get_active_version_manager_by_user_id_as_anonymous_with_access_right_returns_nothing(
        self,
    ):
        """test get active version manager by user id as anonymous with access right returns nothing

        Returns:

        """
        mock_request = create_mock_request(user=self.anonymous_user)
        list_tvm = template_vm_api.get_active_version_manager_by_user_id(
            request=mock_request
        )
        self.assertEqual(list_tvm.count(), 0)

    def test_get_active_version_manager_by_user_id_as_user_returns_user_templates(
        self,
    ):
        """test get active version manager by user id as user returns user templates

        Returns:

        """
        mock_request = create_mock_request(user=self.user1)
        list_tvm = template_vm_api.get_active_version_manager_by_user_id(
            request=mock_request
        )
        for tvm in list_tvm:
            self.assertEqual(tvm.user, str(self.user1.id))

    def test_get_active_version_manager_by_user_id_as_staff_returns_user_templates(
        self,
    ):
        """test get active version manager by user id as staff returns user templates

        Returns:

        """
        mock_request = create_mock_request(user=self.staff_user1)
        list_tvm = template_vm_api.get_active_version_manager_by_user_id(
            request=mock_request
        )
        for tvm in list_tvm:
            self.assertEqual(tvm.user, str(self.staff_user1.id))

    def test_get_active_version_manager_by_user_id_as_superuser_returns_user_templates(
        self,
    ):
        """test get active version manager by user id as superuser returns user templates

        Returns:

        """
        mock_request = create_mock_request(user=self.superuser1)
        list_tvm = template_vm_api.get_active_version_manager_by_user_id(
            request=mock_request
        )
        for tvm in list_tvm:
            self.assertEqual(tvm.user, str(self.superuser1.id))


class TestTemplateGetAllByUserId(IntegrationBaseTestCase):
    """TestTemplateGetAllByUserId"""

    fixture = fixture_template_vm

    def setUp(self):
        """setUp

        Returns:

        """
        self.anonymous_user = create_mock_user(user_id=None, is_anonymous=True)
        self.user1 = create_mock_user(user_id="1")
        self.staff_user1 = create_mock_user(user_id="1", is_staff=True)
        self.superuser1 = create_mock_user(user_id="1", is_superuser=True)
        self.fixture.insert_data()

    def test_get_all_by_user_id_as_anonymous_returns_nothing(self):
        """test get all by user id as anonymous returns nothing

        Returns:

        """
        mock_request = create_mock_request(user=self.anonymous_user)
        list_tvm = template_vm_api.get_all_by_user_id(request=mock_request)
        self.assertEqual(list_tvm.count(), 0)

    @override_settings(CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT=True)
    def test_get_all_by_user_id_as_anonymous_with_access_right_returns_nothing(
        self,
    ):
        """test get all by user id as anonymous with access right returns nothing

        Returns:

        """
        mock_request = create_mock_request(user=self.anonymous_user)
        list_tvm = template_vm_api.get_all_by_user_id(request=mock_request)
        self.assertEqual(list_tvm.count(), 0)

    def test_get_all_by_user_id_as_user_returns_user_templates(self):
        """test get all by user id as user returns user templates

        Returns:

        """
        mock_request = create_mock_request(user=self.user1)
        list_tvm = template_vm_api.get_all_by_user_id(request=mock_request)
        for tvm in list_tvm:
            self.assertEqual(tvm.user, str(self.user1.id))

    def test_get_all_by_user_id_as_staff_returns_user_templates(self):
        """test get all by user id as staff returns user templates

        Returns:

        """
        mock_request = create_mock_request(user=self.staff_user1)
        list_tvm = template_vm_api.get_all_by_user_id(request=mock_request)
        for tvm in list_tvm:
            self.assertEqual(tvm.user, str(self.staff_user1.id))

    def test_get_all_by_user_id_as_superuser_returns_user_templates(self):
        """test get all by user id as superuser returns user templates

        Returns:

        """
        mock_request = create_mock_request(user=self.superuser1)
        list_tvm = template_vm_api.get_all_by_user_id(request=mock_request)
        for tvm in list_tvm:
            self.assertEqual(tvm.user, str(self.superuser1.id))


class TestUpdateTemplatesOrdering(IntegrationBaseTestCase):
    """Test Update Templates Ordering"""

    fixture = fixture_template_vm_ordering

    def setUp(self):
        """setUp

        Returns:

        """
        self.anonymous_user = create_mock_user(user_id=None, is_anonymous=True)
        self.user1 = create_mock_user(user_id="1")
        self.user2 = create_mock_user(user_id="2")
        self.superuser1 = create_mock_user(user_id="1", is_superuser=True)
        self.fixture.insert_data()

    def test_update_user_templates_ordering_as_anonymous_raises_access_control_error(
        self,
    ):
        """test update user templates ordering as anonymous raises acl error

        Returns:x

        """

        # Arrange
        list_templates_ordering = [self.fixture.tvm2.id, self.fixture.tvm1.id]

        # Act # Assert
        with self.assertRaises(AccessControlError):
            template_vm_api._update_template_version_manager_ordering(
                list_templates_ordering,
                create_mock_request(user=self.anonymous_user),
            )

    def test_update_global_templates_ordering_as_anonymous_raises_access_control_error(
        self,
    ):
        """test update global templates ordering as anonymous raises access control error

        Returns:

        """
        # Arrange
        list_templates_ordering = [
            self.fixture.global_tvm2.id,
            self.fixture.global_tvm1.id,
        ]
        # Act # Assert
        with self.assertRaises(AccessControlError):
            template_vm_api._update_template_version_manager_ordering(
                list_templates_ordering,
                create_mock_request(user=self.anonymous_user),
            )

    def test_update_own_templates_ordering_as_user_saves(self):
        """test update own templates ordering as user saves

        Returns:

        """
        # Arrange
        list_templates_ordering = [self.fixture.tvm2.id, self.fixture.tvm1.id]
        # Act
        template_vm_api._update_template_version_manager_ordering(
            list_templates_ordering, create_mock_request(user=self.user1)
        )

    def test_update_other_users_template_ordering_as_user_raises_access_control_error(
        self,
    ):
        """test update other users template ordering as user raises access control error

        Returns:

        """
        # Arrange
        list_templates_ordering = [self.fixture.tvm2.id, self.fixture.tvm1.id]
        # Act # Assert
        with self.assertRaises(AccessControlError):
            template_vm_api._update_template_version_manager_ordering(
                list_templates_ordering, create_mock_request(user=self.user2)
            )

    def test_update_other_users_template_ordering_as_superuser_saves(self):
        """test update other users template ordering as superuser saves

        Returns:

        """
        # Arrange
        list_templates_ordering = [self.fixture.tvm2.id, self.fixture.tvm1.id]
        # Act
        template_vm_api._update_template_version_manager_ordering(
            list_templates_ordering, create_mock_request(user=self.superuser1)
        )

    def test_update_global_template_ordering_as_superuser_saves(self):
        """test update other users template ordering as superuser saves

        Returns:

        """
        # Arrange
        list_templates_ordering = [
            self.fixture.global_tvm2.id,
            self.fixture.global_tvm1.id,
        ]
        # Act
        template_vm_api._update_template_version_manager_ordering(
            list_templates_ordering, create_mock_request(user=self.superuser1)
        )


class TestAccessControlCanWriteList(IntegrationBaseTestCase):
    """Test Access Control Can Write List"""

    fixture = fixture_template_vm_ordering

    def setUp(self):
        """setUp

        Returns:

        """
        self.anonymous_user = create_mock_user(user_id=None, is_anonymous=True)
        self.user1 = create_mock_user(user_id="1")
        self.user2 = create_mock_user(user_id="2")
        self.staff_user1 = create_mock_user(user_id="3", is_staff=True)
        self.superuser1 = create_mock_user(user_id="4", is_superuser=True)
        self.fixture.insert_data()

    def test_access_control_can_write_list_as_anonymous_raises_access_control_error(
        self,
    ):
        """test_access_control_can_write_list_as_anonymous_raises_access_control_error"""

        # Arrange
        template_vm_list = [self.fixture.tvm2.id, self.fixture.tvm1.id]

        # Act # Assert
        with self.assertRaises(AccessControlError):
            access_control.can_write_list(
                template_vm_api._update_template_version_manager_ordering,
                template_vm_list,
                create_mock_request(user=self.anonymous_user),
            )

    def test_access_control_can_write_list_as_owner_returns_function(self):
        """test_access_control_can_write_list_as_owner_returns_function"""

        # Arrange
        template_vm_list = [self.fixture.tvm2.id, self.fixture.tvm1.id]

        # Act
        access_control.can_write_list(
            template_vm_api._update_template_version_manager_ordering,
            template_vm_list,
            create_mock_request(user=self.user1),
        )

    def test_access_control_can_write_list_user_templates_as_user_raises_access_control_error(
        self,
    ):
        """test_access_control_can_write_list_user_templates_as_user_raises_access_control_error"""
        # Arrange
        template_vm_list = [self.fixture.tvm2.id, self.fixture.tvm1.id]

        # Act # Assert
        with self.assertRaises(AccessControlError):
            access_control.can_write_list(
                template_vm_api._update_template_version_manager_ordering,
                template_vm_list,
                create_mock_request(user=self.user2),
            )

    def test_access_control_can_write_list_global_templates_as_user_raises_access_control_error(
        self,
    ):
        """test_access_control_can_write_list_global_templates_as_user_raises_access_control_error"""
        # Arrange
        template_vm_list = [
            self.fixture.global_tvm2.id,
            self.fixture.global_tvm1.id,
        ]

        # Act # Assert
        with self.assertRaises(AccessControlError):
            access_control.can_write_list(
                template_vm_api._update_template_version_manager_ordering,
                template_vm_list,
                create_mock_request(user=self.user2),
            )

    def test_access_control_can_write_list_user_templates_as_staff_raises_access_control_error(
        self,
    ):
        """test_access_control_can_write_list_user_templates_as_staff_raises_access_control_error"""

        # Arrange
        template_vm_list = [self.fixture.tvm2.id, self.fixture.tvm1.id]

        # Act
        with self.assertRaises(AccessControlError):
            access_control.can_write_list(
                template_vm_api._update_template_version_manager_ordering,
                template_vm_list,
                create_mock_request(user=self.staff_user1),
            )

    def test_access_control_can_write_list_global_templates_as_staff_returns_function(
        self,
    ):
        """test_access_control_can_write_list_global_templates_as_staff_returns_functiong"""

        # Arrange
        template_vm_list = [
            self.fixture.global_tvm1.id,
            self.fixture.global_tvm2.id,
        ]

        # Act
        access_control.can_write_list(
            template_vm_api._update_template_version_manager_ordering,
            template_vm_list,
            create_mock_request(user=self.staff_user1),
        )

    def test_access_control_can_write_list_as_superuser_returns_function(self):
        """test_access_control_can_write_list_as_superuser_returns_function"""

        # Arrange
        template_vm_list = [self.fixture.global_tvm2.id, self.fixture.tvm1.id]

        # Act
        access_control.can_write_list(
            template_vm_api._update_template_version_manager_ordering,
            template_vm_list,
            create_mock_request(user=self.superuser1),
        )


class TestUpdateUserTemplateOrdering(IntegrationBaseTestCase):
    """Test Update User Template Ordering"""

    fixture = fixture_template_vm_ordering

    def setUp(self):
        """setUp

        Returns:

        """
        self.anonymous_user = create_mock_user(user_id=None, is_anonymous=True)
        self.user1 = create_mock_user(user_id="1")
        self.user2 = create_mock_user(user_id="2")
        self.superuser1 = create_mock_user(user_id="3", is_superuser=True)
        self.fixture.insert_data()

    def test_update_user_template_ordering_as_owner_updates_templates(
        self,
    ):
        """test_update_user_template_ordering_as_owner_updates_templates

        Returns:

        """
        # Arrange
        template_vm_list = [self.fixture.tvm1.id]

        template_vm_api.update_user_template_ordering(
            template_vm_list, create_mock_request(user=self.user1)
        )

    def test_update_user_template_ordering_as_user_raises_acl_error(
        self,
    ):
        """test_update_user_template_ordering_as_user_raises_acl_error

        Returns:

        """
        # Arrange
        list_templates_ordering = [self.fixture.tvm2.id]

        # Act # Assert
        with self.assertRaises(AccessControlError):
            template_vm_api.update_user_template_ordering(
                list_templates_ordering, create_mock_request(user=self.user2)
            )

    def test_update_user_template_ordering_as_anonymous_raises_acl_error(
        self,
    ):
        """test_update_user_template_ordering_as_anonymous_raises_acl_error

        Returns:

        """
        # Arrange
        list_templates_ordering = [self.fixture.tvm2.id]

        # Act # Assert
        with self.assertRaises(AccessControlError):
            template_vm_api.update_user_template_ordering(
                list_templates_ordering,
                create_mock_request(user=self.anonymous_user),
            )

    def test_update_user_template_ordering_as_superuser_raises_acl_error(
        self,
    ):
        """test_update_user_template_ordering_as_superuser_raises_acl_error

        Returns:

        """
        # Arrange
        list_templates_ordering = [self.fixture.tvm2.id]

        # Act # Assert
        with self.assertRaises(AccessControlError):
            template_vm_api.update_user_template_ordering(
                list_templates_ordering,
                create_mock_request(user=self.superuser1),
            )


class TestUpdateGlobalTemplateOrdering(IntegrationBaseTestCase):
    "Test Update Global Template Ordering"

    fixture = fixture_template_vm_ordering

    def setUp(self):
        """setUp

        Returns:

        """
        self.anonymous_user = create_mock_user(user_id=None, is_anonymous=True)
        self.user1 = create_mock_user(user_id="1")
        self.superuser1 = create_mock_user(
            user_id="3", is_superuser=True, is_staff=True
        )
        self.fixture.insert_data()

    def test_update_global_template_ordering_as_superuser_updates_templates(
        self,
    ):
        """test_update_global_template_ordering_as_superuser_updates_templates

        Returns:

        """
        # Arrange
        template_vm_list = [self.fixture.global_tvm1.id]

        template_vm_api.update_global_template_ordering(
            template_vm_list, request=create_mock_request(user=self.superuser1)
        )

    def test_update_user_template_ordering_as_superuser_raises_acl_error(
        self,
    ):
        """test_update_user_template_ordering_as_superuser_raises_acl_error

        Returns:

        """
        # Arrange
        template_vm_list = [self.fixture.tvm2.id]

        # Act # Assert
        with self.assertRaises(AccessControlError):
            template_vm_api.update_global_template_ordering(
                template_vm_list,
                request=create_mock_request(user=self.superuser1),
            )

    def test_update_global_template_ordering_as_user_raises_acl_error(
        self,
    ):
        """test_update_global_template_ordering_as_user_raises_acl_error

        Returns:

        """
        # Arrange
        list_templates_ordering = [self.fixture.global_tvm1.id]

        # Act # Assert
        with self.assertRaises(AccessControlError):
            template_vm_api.update_global_template_ordering(
                list_templates_ordering,
                request=create_mock_request(user=self.user1),
            )

    def test_update_user_template_ordering_as_anonymous_raises_acl_error(
        self,
    ):
        """test_update_user_template_ordering_as_anonymous_raises_acl_error

        Returns:

        """
        # Arrange
        list_templates_ordering = [self.fixture.global_tvm1.id]

        # Act # Assert
        with self.assertRaises(AccessControlError):
            template_vm_api.update_global_template_ordering(
                list_templates_ordering,
                request=create_mock_request(user=self.anonymous_user),
            )



""" View Only Admin
"""
from django.contrib import admin


class ViewOnlyAdmin(admin.ModelAdmin):
    """ViewOnlyAdmin"""

    def has_add_permission(self, request, obj=None):
        """Prevent from manually adding objects"""
        return False

    def has_change_permission(self, request, obj=None):
        """Prevent from manually updating objects"""
        return False



""" Migrations """
# Generated by Django 3.2 on 2021-12-02 01:51


import django.core.validators
from django.db import migrations, models
import django.db.models.deletion

import core_main_app.components.blob.models
from core_main_app.settings import XSD_UPLOAD_DIR, XSLT_UPLOAD_DIR


class Migration(migrations.Migration):
    """Migration"""

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Data",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("dict_content", models.JSONField(blank=True, null=True)),
                (
                    "title",
                    models.CharField(
                        max_length=200,
                        validators=[
                            django.core.validators.RegexValidator(
                                code="invalid_title",
                                message="Title must not be empty or only whitespaces",
                                regex=".*\\S.*",
                            )
                        ],
                    ),
                ),
                (
                    "xml_file",
                    models.FileField(
                        max_length=250,
                        upload_to=core_main_app.utils.storage.storage.user_directory_path,
                    ),
                ),
                (
                    "checksum",
                    models.CharField(
                        blank=True, default=None, max_length=512, null=True
                    ),
                ),
                (
                    "creation_date",
                    models.DateTimeField(blank=True, default=None, null=True),
                ),
                (
                    "last_modification_date",
                    models.DateTimeField(blank=True, default=None, null=True),
                ),
                (
                    "last_change_date",
                    models.DateTimeField(blank=True, default=None, null=True),
                ),
                ("user_id", models.CharField(max_length=200)),
            ],
            options={
                "verbose_name": "Data",
                "verbose_name_plural": "Data",
            },
        ),
        migrations.CreateModel(
            name="Main",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
            ],
            options={
                "verbose_name": "core_main_app",
                "permissions": (
                    ("publish_data", "Can publish data"),
                    ("publish_blob", "Can publish blob"),
                ),
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="Template",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("is_current", models.BooleanField(default=False)),
                ("is_disabled", models.BooleanField(default=False)),
                (
                    "filename",
                    models.CharField(
                        max_length=200,
                        validators=[
                            django.core.validators.RegexValidator(
                                code="invalid_title",
                                message="Title must not be empty or only whitespaces",
                                regex=".*\\S.*",
                            )
                        ],
                    ),
                ),
                (
                    "file",
                    models.FileField(max_length=250, upload_to=XSD_UPLOAD_DIR),
                ),
                (
                    "checksum",
                    models.CharField(
                        blank=True, default=None, max_length=512, null=True
                    ),
                ),
                (
                    "user",
                    models.CharField(
                        blank=True, default=None, max_length=200, null=True
                    ),
                ),
                ("hash", models.CharField(max_length=200)),
                (
                    "_display_name",
                    models.CharField(blank=True, max_length=200),
                ),
                ("creation_date", models.DateTimeField(auto_now_add=True)),
                ("_cls", models.CharField(default="Template", max_length=200)),
                (
                    "dependencies",
                    models.ManyToManyField(
                        blank=True,
                        default=[],
                        related_name="_core_main_app_template_dependencies_+",
                        to="core_main_app.Template",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="TemplateVersionManager",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "title",
                    models.CharField(
                        max_length=200,
                        unique=True,
                        validators=[
                            django.core.validators.RegexValidator(
                                code="invalid_title",
                                message="Title must not be empty or only whitespaces",
                                regex=".*\\S.*",
                            )
                        ],
                    ),
                ),
                (
                    "user",
                    models.CharField(
                        blank=True, default=None, max_length=200, null=True
                    ),
                ),
                ("is_disabled", models.BooleanField(default=False)),
                (
                    "_cls",
                    models.CharField(default="VersionManager", max_length=200),
                ),
                ("creation_date", models.DateTimeField(auto_now_add=True)),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="WebPage",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("type", models.IntegerField()),
                ("content", models.TextField()),
            ],
        ),
        migrations.CreateModel(
            name="Workspace",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "title",
                    models.CharField(
                        max_length=200,
                        validators=[
                            django.core.validators.RegexValidator(
                                code="invalid_title",
                                message="Title must not be empty or only whitespaces",
                                regex=".*\\S.*",
                            )
                        ],
                    ),
                ),
                (
                    "owner",
                    models.CharField(blank=True, max_length=200, null=True),
                ),
                ("read_perm_id", models.CharField(max_length=200)),
                ("write_perm_id", models.CharField(max_length=200)),
                ("is_public", models.BooleanField(default=False)),
            ],
        ),
        migrations.CreateModel(
            name="XslTransformation",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "name",
                    models.CharField(
                        unique=True,
                        max_length=200,
                        validators=[
                            django.core.validators.RegexValidator(
                                code="invalid_title",
                                message="Title must not be empty or only whitespaces",
                                regex=".*\\S.*",
                            )
                        ],
                    ),
                ),
                (
                    "filename",
                    models.CharField(
                        max_length=200,
                        validators=[
                            django.core.validators.RegexValidator(
                                code="invalid_title",
                                message="Title must not be empty or only whitespaces",
                                regex=".*\\S.*",
                            )
                        ],
                    ),
                ),
                (
                    "file",
                    models.FileField(
                        max_length=250, upload_to=XSLT_UPLOAD_DIR
                    ),
                ),
                (
                    "checksum",
                    models.CharField(
                        blank=True, default=None, max_length=512, null=True
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="TemplateXslRendering",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "default_detail_xslt",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="default_detail_xslt",
                        to="core_main_app.xsltransformation",
                    ),
                ),
                (
                    "list_detail_xslt",
                    models.ManyToManyField(
                        blank=True,
                        default=[],
                        to="core_main_app.XslTransformation",
                    ),
                ),
                (
                    "list_xslt",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="list_xslt",
                        to="core_main_app.xsltransformation",
                    ),
                ),
                (
                    "template",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="core_main_app.template",
                    ),
                ),
            ],
        ),
        migrations.AddField(
            model_name="template",
            name="version_manager",
            field=models.ForeignKey(
                default=None,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="core_main_app.templateversionmanager",
            ),
        ),
        migrations.CreateModel(
            name="DatabaseLockObject",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("user_id", models.CharField(max_length=200)),
                ("lock_date", models.DateTimeField()),
                (
                    "object",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="core_main_app.data",
                    ),
                ),
            ],
        ),
        migrations.AddField(
            model_name="data",
            name="template",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                to="core_main_app.template",
            ),
        ),
        migrations.AddField(
            model_name="data",
            name="workspace",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="core_main_app.workspace",
            ),
        ),
        migrations.CreateModel(
            name="Blob",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "filename",
                    models.CharField(
                        max_length=200,
                        validators=[
                            django.core.validators.RegexValidator(
                                code="invalid_title",
                                message="Filename must not be empty or only whitespaces",
                                regex=".*\\S.*",
                            )
                        ],
                    ),
                ),
                ("user_id", models.CharField(max_length=200)),
                (
                    "blob",
                    models.FileField(
                        null=True,
                        max_length=250,
                        upload_to=core_main_app.utils.storage.storage.user_directory_path,
                    ),
                ),
                (
                    "checksum",
                    models.CharField(
                        blank=True, default=None, max_length=512, null=True
                    ),
                ),
                ("creation_date", models.DateTimeField(auto_now_add=True)),
                (
                    "workspace",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="core_main_app.workspace",
                    ),
                ),
            ],
        ),
        migrations.AddIndex(
            model_name="data",
            index=models.Index(
                fields=[
                    "title",
                    "last_modification_date",
                    "template",
                    "user_id",
                ],
                name="core_main_a_title_3e9383_idx",
            ),
        ),
    ]



""" Integration Test for User
"""

from tests.components.user.fixtures.fixtures import UserFixtures
from django.contrib.auth.models import User
from core_main_app.components.user import api as user_api
from core_main_app.utils.integration_tests.integration_base_transaction_test_case import (
    IntegrationTransactionTestCase,
)


class TestUserGetActiveUsers(IntegrationTransactionTestCase):
    """Test User Get Active Users"""

    def test_get_active_users_returns_empty_list(self):
        """test get active users returns empty list

        Returns:

        """

        # Assert
        result = user_api.get_active_users()
        self.assertEqual(len(result), 0)

    def test_get_active_users_returns_list_users(self):
        """test get active users returns list users

        Returns:

        """
        UserFixtures().create_user(username="username")
        # Assert
        result = user_api.get_active_users()
        self.assertEqual(len(result), 1)
        self.assertTrue(user.is_active() for user in result)


class TestUserGetUserByUsername(IntegrationTransactionTestCase):
    """Test User Get User By Username"""

    def test_get_user_by_username_raises_error_when_does_not_exist(self):
        """test get user by username raises error when does not exist

        Returns:

        """

        # Assert
        with self.assertRaises(User.DoesNotExist):
            user_api.get_user_by_username("empty")

    def test_get_user_by_username_returns_user(self):
        """test get user by username returns user

        Returns:

        """
        user = UserFixtures().create_user(username="username")
        # Act
        result = user_api.get_user_by_username("username")
        # Assert
        self.assertIsInstance(result, User)
        self.assertEqual(result.username, user.username)


class TestUserCreateAndSaveUser(IntegrationTransactionTestCase):
    """Test User Create And Save User"""

    def test_create_and_save_user_creates_user(self):
        """test create and save user creates user

        Returns:

        """

        # Act
        result = user_api.create_and_save_user(
            "username", "password", "firstname", "lastname", "email"
        )
        # Assert
        self.assertIsInstance(result, User)
        self.assertEqual(result.username, "username")


class TestUserUpsert(IntegrationTransactionTestCase):
    """Test User Upsert"""

    def test_upsert_user_creates_user(self):
        """test upsert creates user

        Returns:

        """

        # Act
        result = user_api.upsert(User.objects.create_user(username="username"))
        # Assert
        self.assertIsInstance(result, User)
        self.assertEqual(result.username, "username")

    def test_upsert_and_save_user_updates_user(self):
        """test upsert updates user

        Returns:

        """
        # Arrange
        user = user_api.create_and_save_user(
            "username", "password", "firstname", "lastname", "email"
        )
        user.username = "new_username"
        # Act
        result = user_api.upsert(user)
        # Assert
        self.assertIsInstance(result, User)
        self.assertEqual(result.username, "new_username")


class TestUserGetAllUsersExceptList(IntegrationTransactionTestCase):
    """Test User Get All Users Except List"""

    def test_get_all_users_except_list_returns_empty_list(self):
        """test get all users except list returns empty list

        Returns:

        """
        # Arrange
        user = UserFixtures().create_user(username="username")
        # Act
        result = user_api.get_all_users_except_list([user])
        # Assert
        self.assertEqual(len(result), 0)

    def test_get_all_users_except_list_returns_list(self):
        """test get all users except list returns list

        Returns:

        """
        # Arrange
        user1 = UserFixtures().create_user(username="username1")
        user2 = UserFixtures().create_user(username="username2")
        user3 = UserFixtures().create_user(username="username3")

        # Act
        result = user_api.get_all_users_except_list([user1])
        # Assert
        self.assertEqual(len(result), 2)
        self.assertNotIn(user1, result)
        self.assertIn(user2, result)
        self.assertIn(user3, result)


class TestUserGetAllUsersExceptListId(IntegrationTransactionTestCase):
    """Test User Get All Users Except List Id"""

    def test_get_all_users_except_list_id_returns_empty_list(self):
        """test get all users except list id returns empty list

        Returns:

        """
        # Arrange
        user = UserFixtures().create_user(username="username")
        # Act
        result = user_api.get_all_users_except_list_id([user.id])
        # Assert
        self.assertEqual(len(result), 0)

    def test_get_all_users_except_list_id_returns_list(self):
        """test get all users except list id returns list

        Returns:

        """
        # Arrange
        user1 = UserFixtures().create_user(username="username1")
        user2 = UserFixtures().create_user(username="username2")
        user3 = UserFixtures().create_user(username="username3")

        # Act
        result = user_api.get_all_users_except_list_id([user1.id])
        # Assert
        self.assertEqual(len(result), 2)
        self.assertNotIn(user1, result)
        self.assertIn(user2, result)
        self.assertIn(user3, result)


class TestUserGetAllUsersByListId(IntegrationTransactionTestCase):
    """Test User Get All Users By List Id"""

    def test_get_all_users_by_list_id_returns_empty_list(self):
        """test get all users By list id returns empty list

        Returns:

        """
        # Act
        result = user_api.get_all_users_by_list_id([])
        # Assert
        self.assertEqual(len(result), 0)

    def test_get_all_users_by_list_id_returns_valid_user_id_only(self):
        """test get all users By list id returns valid user id only

        Returns:

        """
        # Arrange
        user = UserFixtures().create_user(username="username")
        # Act
        result = user_api.get_all_users_by_list_id([-1, user.id])
        # Assert
        self.assertEqual(len(result), 1)
        self.assertTrue(user in result)

    def test_get_all_users_by_list_id_returns_list(self):
        """test get all users by list id returns list

        Returns:

        """
        # Arrange
        user1 = UserFixtures().create_user(username="username1")
        user2 = UserFixtures().create_user(username="username2")
        user3 = UserFixtures().create_user(username="username3")

        # Act
        result = user_api.get_all_users_by_list_id([user1.id, user2.id])
        # Assert
        self.assertEqual(len(result), 2)
        self.assertIn(user1, result)
        self.assertIn(user2, result)
        self.assertNotIn(user3, result)


class TestUserGetIdUsernameDict(IntegrationTransactionTestCase):
    """Test User Get Id Username Dict"""

    def test_get_id_username_dict_returns_empty_list(self):
        """test get id username dict returns empty list

        Returns:

        """
        # Act
        result = user_api.get_id_username_dict([])
        # Assert
        self.assertEqual(len(result), 0)

    def test_get_id_username_dict_returns_list(self):
        """test get id username dict returns list

        Returns:

        """
        # Arrange
        user1 = UserFixtures().create_user(username="username1")
        user2 = UserFixtures().create_user(username="username2")
        UserFixtures().create_user(username="username3")

        # Act
        result = user_api.get_id_username_dict([user1, user2])
        # Assert
        self.assertEqual(len(result), 2)



""" Utils to provide regex validation
"""
import re

from django.core.exceptions import ValidationError
from django.core.files.utils import validate_file_name

from core_main_app.commons.regex import NOT_EMPTY_OR_WHITESPACES


def not_empty_or_whitespaces(value):
    """Check that the value is not an empty string or only white spaces

    Args:
        value:

    Returns:

    """
    pattern = re.compile(NOT_EMPTY_OR_WHITESPACES)
    if not pattern.match(value):
        raise ValidationError("Value can not be empty")


def validate_alphanum(name):
    """Validate alpha-numerical characters

    Args:
        name:

    Returns:

    """
    if re.match(r"^[a-zA-Z][a-zA-Z0-9]+$", name) is None:
        raise ValidationError(
            "Name should only contains alpha-numerical characters."
        )


def validate_filename(filename):
    """Validate filename

    Args:
        filename:

    Returns:

    """
    # Run validate filename from Django
    validate_file_name(filename)
    # Regex check filename has extension
    if re.match(r"^[a-zA-Z0-9-_]+\.[a-z0-9]{2,4}$", filename) is None:
        raise ValidationError("Invalid filename.")



""" Custom admin site for the Workspace model
"""
from django.contrib import admin


class CustomWorkspaceAdmin(admin.ModelAdmin):
    """CustomWorkspaceAdmin"""

    readonly_fields = ["title"]
    exclude = ["read_perm_id", "write_perm_id"]

    def has_add_permission(self, request, obj=None):
        """Prevent from manually adding Workspaces"""
        return False



""" Custom admin site for the Blob model
"""
from django.contrib import admin


class CustomBlobAdmin(admin.ModelAdmin):
    """CustomBlobAdmin"""

    readonly_fields = ["checksum", "blob"]

    def has_add_permission(self, request, obj=None):
        """Prevent from manually adding Blobs"""
        return False



""" Blob utils test class
"""
from unittest import TestCase
from unittest.mock import patch

from core_main_app.commons.exceptions import CoreError
from core_main_app.utils.file import (
    get_base_64_content_from_response,
    get_byte_size_from_string,
)


class TestFileUtilsGetBase64ContentFromResponse(TestCase):
    """TestFileUtilsGetBase64ContentFromResponse"""

    def test_get_base_64_content_from_response_return_base_64_encoded_string(
        self,
    ):
        """test get base 64 content from response return base 64 encoded string

        Returns:

        """
        # Arrange
        response = MockResponse()
        # Act
        result = get_base_64_content_from_response(response)
        # Assert
        self.assertEqual(True, isinstance(result, str))

    def test_get_base_64_content_from_response_raise_core_exception_if_encoding_fails(
        self,
    ):
        """test get base 64 content from response raise core exception if encoding fails

        Returns:

        """
        # Arrange
        response = MockResponse()
        response.content = "this_is_a_string and it will fail"
        # Assert
        with self.assertRaises(CoreError):
            # Act
            get_base_64_content_from_response(response)

    @patch("base64.b64decode")
    def test_get_base_64_content_from_response_raise_core_exception_if_ascii_decoding_fails(
        self, mock_b64decode
    ):
        """test get base 64 content from response raise core exception if ascii decoding fails

        Args:
            mock_b64decode:

        Returns:

        """
        # Arrange
        response = MockResponse()
        response.content = "this_is_a_string and it will fail"
        mock_b64decode.side_effect = Exception()
        # Assert
        with self.assertRaises(CoreError):
            # Act
            get_base_64_content_from_response(response)


class MockResponse:
    """MockResponse"""

    content = b"my_string"


class TestGetByteSizeFromString(TestCase):
    """TestGetByteSizeFromString"""

    def test_get_byte_size_from_string_returns_int(
        self,
    ):
        """test_get_byte_size_from_string_returns_int

        Returns:

        """
        # Arrange
        result = get_byte_size_from_string("test")
        self.assertTrue(isinstance(result, int))
        self.assertTrue(result > 0)

    def test_get_byte_size_from_empty_string_returns_zero(
        self,
    ):
        """test_get_byte_size_from_string_returns_int

        Returns:

        """
        # Arrange
        result = get_byte_size_from_string("")
        self.assertTrue(result == 0)



""" Tests of the web page API
"""

from unittest.case import TestCase
from unittest.mock import Mock, patch

from core_main_app.commons.enums import WEB_PAGE_TYPES
from core_main_app.commons.exceptions import ApiError
from core_main_app.commons.exceptions import DoesNotExist
from core_main_app.components.web_page import api as web_page_api
from core_main_app.components.web_page.models import WebPage


class TestsWebPageApiGet(TestCase):
    """TestsWebPageApiGet"""

    @patch("core_main_app.components.web_page.models.WebPage.get_by_type")
    def test_web_page_get_login(self, mock_get_web_page_by_type):
        """test web page get login

        Args:
            mock_get_web_page_by_type:

        Returns:

        """
        # Arrange
        content = "content web page login"
        mock_get_web_page_by_type.return_value = _create_mock_web_page(
            WEB_PAGE_TYPES["login"], content
        )
        # Act
        result = web_page_api.get("login")
        # Assert
        self.assertEqual("content web page login", result.content)

    @patch("core_main_app.components.web_page.models.WebPage.get_by_type")
    def test_web_page_get_not_in_database_return_none(
        self, mock_get_web_page_by_type
    ):
        """test web page get not in database return none

        Args:
            mock_get_web_page_by_type:

        Returns:

        """
        # Arrange
        mock_get_web_page_by_type.side_effect = DoesNotExist("")
        # Act
        result = web_page_api.get("login")
        # Assert
        self.assertEqual(None, result)

    def test_web_page_get_with_wrong_type_return_none(self):
        """test web page get with wrong type return none

        Returns:

        """
        # Act
        page = web_page_api.get("wrong_type")
        # Assert
        self.assertEqual(None, page)


class TestsWebPageApiDeleteByType(TestCase):
    """Tests Web Page Api Delete By Type"""

    @patch("core_main_app.components.web_page.models.WebPage.delete_by_type")
    def test_web_page_delete_by_type_raises_error_when_does_not_exist(
        self, mock_delete_by_type
    ):
        """test web page delete by type raises error when does not exist

        Returns:

        """
        # Arrange
        mock_delete_by_type.side_effect = DoesNotExist("")
        # Act # Assert
        with self.assertRaises(DoesNotExist):
            web_page_api.delete_by_type(-1)

    @patch("core_main_app.components.web_page.models.WebPage.delete_by_type")
    def test_web_page_delete_by_type_deletes_web_page(
        self, mock_delete_by_type
    ):
        """test web page delete by type deletes web page

        Args:
            mock_delete_by_type:

        Returns:

        """
        # Arrange
        mock_delete_by_type.return_value = None
        # Act
        result = web_page_api.delete_by_type(1)
        # Assert
        self.assertEqual(result, None)


class TestsWebPageApiUpsert(TestCase):
    """TestsWebPageApiUpsert"""

    def test_web_page_upsert_type_does_not_exist(self):
        """test web page upsert type does not exist

        Returns:

        """
        # Arrange
        web_page = WebPage(type=4, content="test")
        # Act # Assert
        with self.assertRaises(ApiError):
            web_page_api.upsert(web_page)

    @patch("core_main_app.components.web_page.models.WebPage.save")
    def test_web_page_upsert_type_exist(self, mock_save):
        """test web page upsert type exist

        Args:
            mock_save:

        Returns:

        """
        # Arrange
        web_page_type = WEB_PAGE_TYPES["login"]
        content = "content"
        web_page = WebPage(type=web_page_type, content=content)
        mock_save.return_value = WebPage(type=web_page_type, content=content)
        # Act
        result = web_page_api.upsert(web_page)
        # Assert
        self.assertEqual(content, result.content)

    @patch("core_main_app.components.web_page.models.WebPage.delete_by_type")
    def test_web_page_upsert_type_deletes_when_content_is_empty(
        self, mock_delete_by_type
    ):
        """test web page upsert type deletes when content is empty

        Args:
            mock_delete_by_type:

        Returns:

        """
        # Arrange
        web_page_type = WEB_PAGE_TYPES["login"]
        content = ""
        web_page = WebPage(type=web_page_type, content=content)
        mock_delete_by_type.return_value = None
        # Act
        result = web_page_api.upsert(web_page)
        # Assert
        self.assertEqual(result, None)


def _create_mock_web_page(page_type=-1, content="content"):
    """create mock web page

    Args:
        page_type:
        content:

    Returns:

    """
    mock_web_page = Mock(spec=WebPage)
    mock_web_page.type = page_type
    mock_web_page.content = content
    return mock_web_page



""" Mock classes for use in unit tests.
"""
from unittest.mock import Mock


class MockQuerySet(Mock):
    """Mock Query Set"""

    item_list = []

    def count(self):
        """Count the number of items in the queryset"""
        return len(self.item_list)

    def __getitem__(self, item):
        """Iter items"""
        return self.item_list[item]



""" Rest views
"""
import importlib_metadata
from django.conf import settings
from django.db import connection
from rest_framework import status
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response
from rest_framework.views import APIView

from core_main_app.utils.databases.backend import (
    uses_postgresql_backend,
    uses_sqlite3_backend,
)
from core_main_app.utils.databases.mongo import MONGO_CLIENT


class CoreSettings(APIView):
    """Get Core Settings"""

    permission_classes = (IsAuthenticated,)

    def get(self, request):
        """Get Core setting

        Args:

            request: HTTP request

        Returns:

            - code: 200
              content: Settings
            - code: 500
              content: Internal server error
        """
        try:
            # Get version of core main application
            try:
                core_version = importlib_metadata.version("core_main_app")
            except importlib_metadata.PackageNotFoundError:
                core_version = None

            # Get version of MongoDB server
            mongodb_version = (
                (MONGO_CLIENT.database.client.server_info()["version"])
                if MONGO_CLIENT
                else None
            )

            # Get PSQL version
            if uses_postgresql_backend():
                database_info = (
                    "PostgreSQL",
                    connection.cursor().connection.server_version,
                )
            # Get SQLite version
            elif uses_sqlite3_backend():
                database_info = (
                    "SQLite3",
                    connection.cursor().db.Database.version,
                )
            else:
                database_info = (None, None)

            response_dict = {
                "core_version": core_version,
                "database": {
                    "engine": database_info[0],
                    "version": database_info[1],
                },
                "mongodb": {
                    "data_indexing": settings.MONGODB_INDEXING,
                    "version": mongodb_version,
                },
            }
            return Response(response_dict, status=status.HTTP_200_OK)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(
                content, status=status.HTTP_500_INTERNAL_SERVER_ERROR
            )



""" Constant for admin site
"""
from django.contrib import admin
from django.conf import settings


def get_base_model_admin_class(model):
    """Return base model admin class (using simple history or not)

    Args:
        model:

    Returns:

    """
    django_simple_history_models = getattr(
        settings, "DJANGO_SIMPLE_HISTORY_MODELS", None
    )
    if django_simple_history_models and isinstance(
        django_simple_history_models, list
    ):
        # if model found in settings
        if model in django_simple_history_models:
            from simple_history.admin import SimpleHistoryAdmin

            # use simple history
            return SimpleHistoryAdmin
    # otherwise use django model admin
    return admin.ModelAdmin


def register_simple_history_models(django_simple_history_models):
    """Register models for Django simple history

    Args:
        django_simple_history_models:

    Returns:

    """
    if django_simple_history_models and isinstance(
        django_simple_history_models, list
    ):
        from simple_history import register

        if "Data" in django_simple_history_models:
            from core_main_app.components.data.models import Data

            register(Data)



"""Unit tests for xsl transformation rest api
"""
import unittest
from unittest.mock import patch

from django.test.testcases import SimpleTestCase
from rest_framework import status

from core_main_app.components.xsl_transformation.models import (
    XslTransformation,
)
from core_main_app.rest.xsl_transformation import views as xsl_views
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import RequestMock


class TestGetAllXslTransformation(SimpleTestCase):
    """TestGetAllXslTransformation"""

    def setUp(self):
        """setUp

        Returns:

        """
        super().setUp()
        self.data = None

    @patch.object(XslTransformation, "get_all")
    def test_get_all_xsl_document_returns_status_403_with_no_permission_needed(
        self, mock_get_all
    ):
        """test_get_all_xsl_document_returns_status_403_with_no_permission_needed

        Args:
            mock_get_all:

        Returns:

        """
        # Arrange
        user = create_mock_user("0")

        # Act
        response = RequestMock.do_request_get(
            xsl_views.XslTransformationList.as_view(), user, self.data
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(XslTransformation, "get_all")
    def test_get_all_xsl_document_returns_status_200_with_staff_permission(
        self, mock_get_all
    ):
        """test_get_all_xsl_document_returns_status_200_with_staff_permission

        Args:
            mock_get_all:

        Returns:

        """
        # Arrange
        user = create_mock_user("0", True)

        # Act
        response = RequestMock.do_request_get(
            xsl_views.XslTransformationList.as_view(), user, self.data
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestPostXslTransformation(SimpleTestCase):
    """TestPostXslTransformation"""

    def setUp(self):
        """setUp

        Returns:

        """
        super().setUp()
        self.data = None

    @patch.object(XslTransformation, "save_object")
    def test_post_xsl_document_returns_status_403_if_user_is_not_admin(
        self, mock_save
    ):
        """test_post_xsl_document_returns_status_403_if_user_is_not_admin

        Args:
            mock_save:

        Returns:

        """
        # Arrange
        user = create_mock_user("0")

        # Act
        response = RequestMock.do_request_post(
            xsl_views.XslTransformationList.as_view(), user, self.data
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @unittest.skip("Not working on Jenkins")
    @patch.object(XslTransformation, "save_object")
    def test_post_xsl_document_returns_status_400_if_data_are_not_valid_with_admin_user(
        self, mock_save
    ):
        """test_post_xsl_document_returns_status_400_if_data_are_not_valid_with_admin_user

        Args:
            mock_save:

        Returns:

        """
        # Arrange
        user = create_mock_user("0", True, True)

        # Act
        response = RequestMock.do_request_post(
            xsl_views.XslTransformationList.as_view(), user, self.data
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    @unittest.skip("Not working on Jenkins")
    @patch.object(XslTransformation, "save_object")
    def test_post_xsl_document_returns_status_201_if_data_are_valid_with_admin_user(
        self, mock_save
    ):
        """test_post_xsl_document_returns_status_201_if_data_are_valid_with_admin_user

        Args:
            mock_save:

        Returns:

        """
        # Arrange
        user = create_mock_user("0", True, True)
        self.data = {
            "name": "name",
            "filename": "filename.xsd",
            "content": '<?xml version="1.0" encoding="UTF-8"?>'
            '<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">'
            "<xsl:template></xsl:template></xsl:stylesheet>",
        }

        # Act
        response = RequestMock.do_request_post(
            xsl_views.XslTransformationList.as_view(), user, self.data
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)



# Generated by Django 4.2.18 on 2023-08-24 14:55

from django.db import migrations, models

import core_main_app.components.user_preferences.models
import core_main_app.utils.storage.storage


class Migration(migrations.Migration):

    dependencies = [
        ("core_main_app", "0006_blob_metadata"),
    ]

    operations = [
        migrations.CreateModel(
            name="UserPreferences",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("user_id", models.CharField(max_length=200, unique=True)),
                (
                    "timezone",
                    models.CharField(
                        blank=True,
                        max_length=200,
                        null=True,
                        validators=[
                            core_main_app.components.user_preferences.models.validate_timezone
                        ],
                    ),
                ),
            ],
            options={
                "verbose_name": " User Preferences",
                "verbose_name_plural": " User Preferences",
            },
        ),
    ]



""" View builder for view data page.
"""
import logging

from django.conf import settings as conf_settings

from core_main_app.commons import exceptions
from core_main_app.components.template_xsl_rendering import (
    api as template_xsl_rendering_api,
)

logger = logging.getLogger(__name__)


def build_page(
    data_object, display_admin_version=False, display_download_options=False
):
    """Generic page building data

    Args:
        data_object:
        display_admin_version:
        display_download_options:

    Returns:
    """
    page_info = {
        "error": None,
        "context": {},
        "assets": {},
        "modals": [],
    }

    try:
        (
            display_xslt_selector,
            template_xsl_rendering,
            xsl_transformation_id,
        ) = xslt_selector(
            _get_field(_get_field(data_object, "template"), "id")
        )

        page_info["context"] = {
            "data": data_object,
            "share_pid_button": False,
            "template_xsl_rendering": template_xsl_rendering,
            "xsl_transformation_id": xsl_transformation_id,
            "can_display_selector": display_xslt_selector,
            "display_download_options": display_download_options,
            "page_title": _get_field(data_object, "title"),
        }

        page_info["assets"] = {
            "js": [
                {
                    "path": "core_main_app/common/js/XMLTree.js",
                    "is_raw": False,
                },
                {
                    "path": "core_main_app/user/js/data/detail.js",
                    "is_raw": False,
                },
                {
                    "path": "core_main_app/user/js/data/change_display.js",
                    "is_raw": False,
                },
            ],
            "css": ["core_main_app/common/css/XMLTree.css"],
        }

        if display_download_options:
            page_info["assets"]["js"].extend(
                [
                    {
                        "path": "core_main_app/common/js/data_detail.js",
                        "is_raw": False,
                    },
                    {
                        "path": "core_main_app/common/js/modals/download.js",
                        "is_raw": False,
                    },
                ]
            )
            page_info["modals"].append(
                "core_main_app/common/modals/download-options.html"
            )

        if "core_file_preview_app" in conf_settings.INSTALLED_APPS:
            page_info["assets"]["js"].extend(
                [
                    {
                        "path": "core_file_preview_app/user/js/file_preview.js",
                        "is_raw": False,
                    }
                ]
            )
            page_info["assets"]["css"].append(
                "core_file_preview_app/user/css/file_preview.css"
            )
            page_info["modals"].append(
                "core_file_preview_app/user/file_preview_modal.html"
            )

        if "core_linked_records_app" in conf_settings.INSTALLED_APPS:
            from core_linked_records_app.system.pid_settings import (
                api as pid_settings_system_api,
            )

            if pid_settings_system_api.get().auto_set_pid:
                page_info["context"]["share_pid_button"] = True
                page_info["assets"]["js"].extend(
                    [
                        {
                            "path": "core_main_app/user/js/sharing_modal.js",
                            "is_raw": False,
                        },
                        {
                            "path": "core_linked_records_app/user/js/sharing/data_detail.js",
                            "is_raw": False,
                        },
                    ]
                )
                page_info["assets"]["css"].append(
                    "core_main_app/common/css/share_link.css"
                )
                page_info["modals"].append(
                    "core_linked_records_app/user/sharing/data_detail/modal.html"
                )
    except exceptions.DoesNotExist:
        page_info["error"] = "Data not found"
    except exceptions.ModelError:
        page_info["error"] = "Model error"
    except Exception as exception:
        page_info["error"] = str(exception)
    finally:
        return page_info


def _get_field(data_object, field):
    """Get value of field from object or dict

    Args:
        data_object:
        field:

    Returns:

    """
    if field not in ["title", "template", "id"]:
        return None
    # If data is None, title is None
    if not data_object:
        return None
    # If data is a Data, return data.field
    if hasattr(data_object, field):
        return getattr(data_object, field)
    # If data is a dict, return data['field']
    if isinstance(data_object, dict):
        return data_object.get(field, None)
    return None


def xslt_selector(template_id):
    """Setup Xslt Selector

    Args:
        template_id

    Return:

    """

    display_xslt_selector = True
    try:
        template_xsl_rendering = template_xsl_rendering_api.get_by_template_id(
            template_id
        )
        xsl_transformation_id = (
            template_xsl_rendering.default_detail_xslt.id
            if template_xsl_rendering.default_detail_xslt
            else None
        )
        if template_xsl_rendering.list_detail_xslt.count() == 0 or (
            template_xsl_rendering.default_detail_xslt is not None
            and template_xsl_rendering.list_detail_xslt.count() == 1
        ):
            display_xslt_selector = False

    except Exception as exception:
        logger.warning(
            "An exception occurred when retrieving XSLT: %s",
            str(exception),
        )
        display_xslt_selector = False
        template_xsl_rendering = None
        xsl_transformation_id = None

    return display_xslt_selector, template_xsl_rendering, xsl_transformation_id


def render_page(request, render_function, template, context):
    """render page

    Args:
        request:
        render_function:
        template:
        context:

    Returns:

    """
    if context["error"] is not None:
        return render_function(
            request,
            "core_main_app/common/commons/error.html",
            context={"error": context["error"]},
        )

    return render_function(
        request,
        template,
        context=context["context"],
        assets=context["assets"],
        modals=context["modals"],
    )



""" Custom admin site for the Template Version Manager model
"""
from django.contrib import admin


class CustomTemplateVersionManagerAdmin(admin.ModelAdmin):
    """CustomTemplateVersionManagerAdmin"""

    exclude = ["_cls"]
    search_fields = ["title"]
    list_filter = ["title", "user"]
    list_display = ["title", "user", "display_rank", "creation_date"]

    def has_add_permission(self, request, obj=None):
        """Prevent from manually adding Template version managers"""
        return False



""" Data tasks
"""

import logging

from celery import shared_task
from celery.result import AsyncResult

from core_main_app.access_control.exceptions import AccessControlError
from core_main_app.components.data import api as data_api
from core_main_app.components.user import api as user_api
from core_main_app.components.xsl_transformation import (
    api as xsl_transformation_api,
)
from core_main_app.system import api as system_api

logger = logging.getLogger(__name__)


@shared_task
def async_migration_task(data_list, xslt_id, template_id, user_id, migrate):
    """Async task which perform a migration / validation of the data list for the given target template id

    Args:
        data_list:
        xslt_id:
        template_id:
        user_id:
        migrate: (boolean) Perform the migration

    Return:
        {"valid": ["id"...], "wrong": ["id"...]}
    """
    success = []
    errors = []
    current_progress = 0
    total_data = len(data_list)

    try:
        user = user_api.get_user_by_id(user_id)
        # check user status
        if not (user.is_staff or user.is_superuser):
            raise AccessControlError("Only admin user can migrate data.")

        target_template = system_api.get_template_by_id(template_id)

        # get xsl transformation if selected
        if xslt_id is not None:
            xslt = xsl_transformation_api.get_by_id(str(xslt_id))

        for data_id in data_list:
            data = data_api.get_by_id(data_id, user=user)
            # modify the data temporarily with the new targeted template
            data.template = target_template

            if xslt_id is not None:
                # modify the xml content temporarily with the transformed data content
                data.xml_content = xsl_transformation_api.xsl_transform(
                    data.xml_content, xslt.name
                )

            try:
                # save the new template for the data if the migration is True
                if migrate:
                    system_api.upsert_data(data)
                else:
                    # check if the data is valid
                    data_api.check_xml_file_is_valid(data)

                success.append(str(data.id))
            except Exception:
                errors.append(str(data.id))
            finally:
                # increase the current progress and update the task state
                current_progress += 1
                async_migration_task.update_state(
                    state="PROGRESS",
                    meta={"current": current_progress, "total": total_data},
                )
    except Exception as exception:
        async_migration_task.update_state(
            state="ABORT",
            meta={"current": current_progress, "total": total_data},
        )
        raise Exception(f"Something went wrong: {str(exception)}")

    return {"valid": success, "wrong": errors}


@shared_task
def async_template_migration_task(
    templates, xslt_id, target_template_id, user_id, migrate
):
    """Async task which perform a migration / validation of all the data which belong to the given template id list

    Args:
        templates:
        xslt_id:
        target_template_id:
        user_id
        migrate: (boolean) Perform the migration

    Return:
        {"valid": <number>, "wrong": <number>}
    """
    # get the data list to check
    current_data_progress = 0
    current_template_progress = -1
    total_data = 0
    total_template = len(templates)
    success = []
    error = []
    try:
        if target_template_id and total_template > 0:
            # get the user
            user = user_api.get_user_by_id(user_id)
            # check user status
            if not (user.is_staff or user.is_superuser):
                raise AccessControlError("Only admin user can migrate data.")
            # get the target template
            target_template = system_api.get_template_by_id(target_template_id)
            # get xsl transformation if selected
            if xslt_id is not None:
                xslt = xsl_transformation_api.get_by_id(str(xslt_id))

            for template_id in templates:

                # increase the number of processed template
                current_template_progress += 1
                # rest de number of data
                current_data_progress = 0

                # get a QuerySet of all the data with the given template
                data_list = system_api.get_all_by_template(template_id)

                total_data = data_list.count()

                for data in data_list.all():
                    # modify the data temporarily with the new targeted template
                    data.template = target_template

                    if xslt_id is not None:
                        # modify the xml content temporarily with the transformed data content
                        data.xml_content = (
                            xsl_transformation_api.xsl_transform(
                                data.xml_content, xslt.name
                            )
                        )

                    # check if the data is valid
                    try:
                        # save the new template for the data if the migration is True
                        if migrate:
                            system_api.upsert_data(data)
                        else:
                            data_api.check_xml_file_is_valid(data)

                        success.append(str(data.id))
                    except Exception:
                        error.append(str(data.id))
                    finally:
                        # increase the current progress and update the task state
                        current_data_progress += 1
                        async_template_migration_task.update_state(
                            state="PROGRESS",
                            meta={
                                "template_current": current_template_progress,
                                "template_total": total_template,
                                "data_current": current_data_progress,
                                "data_total": total_data,
                            },
                        )

            return {"valid": success, "wrong": error}

        else:
            async_template_migration_task.update_state(
                state="ABORT",
                meta={
                    "template_current": current_template_progress,
                    "template_total": total_template,
                    "data_current": current_data_progress,
                    "data_total": total_data,
                },
            )
            raise Exception(
                "Wrong template id."
                if not target_template_id
                else "Please provide template id."
            )
    except Exception as exception:
        async_template_migration_task.update_state(
            state="ABORT",
            meta={
                "template_current": current_template_progress,
                "template_total": total_data,
                "data_current": current_data_progress,
                "data_total": total_data,
            },
        )
        raise Exception(f"Something went wrong: {str(exception)}")


def get_task_progress(task_id):
    """Get task status for the given task id

    Args:
        task_id:

    Return:
        {
            'state': PENDING | PROGRESS | SUCCESS,
            'details': result (for SUCCESS) | null (for PENDING) | { PROGRESS info }
        }
    """
    result = AsyncResult(task_id)
    response_data = {
        "state": result.state,
        "details": result.info,
    }
    return response_data


def get_task_result(task_id):
    """Get task result for the given task id

    Args:
        task_id:

    Return: {
                "valid": ["data_id_1", "data_id_2" ...],
                "wrong": ["data_id_3", "data_id_4" ...]
            }
    """
    result = AsyncResult(task_id).result
    return result


@shared_task
def index_mongo_data(data_id):
    """Index a data in MongoDB"""
    try:
        data = system_api.get_data_by_id(data_id)
        try:
            from core_main_app.components.mongo.models import MongoData

            mongo_data = MongoData.init_mongo_data(data)
            mongo_data.save()
        except Exception as exception:
            logger.error(
                f"ERROR : An error occurred while indexing data : {str(exception)}"
            )
    except Exception as exception:
        logger.error(
            f"ERROR : An error occurred while indexing data : {str(exception)}"
        )


@shared_task
def update_mongo_data_user(data_ids, user_id):
    """Update user id of all data in list

    Args:
        data_ids:
        user_id:

    Returns:

    """
    try:
        from core_main_app.components.mongo.models import MongoData

        for data_id in data_ids:
            mongo_data = MongoData.objects.get(pk=data_id)
            mongo_data.user_id = user_id
            mongo_data.save()
    except Exception as exception:
        logger.error(
            f"ERROR : An error occurred while updating data owner : {str(exception)}"
        )


@shared_task
def update_mongo_data_workspace(data_ids, workspace_id):
    """Update workspace id of all data in list
    Args:
        data_ids:
        workspace_id:

    Returns:

    """

    try:
        from core_main_app.components.mongo.models import MongoData

        for data_id in data_ids:
            mongo_data = MongoData.objects.get(pk=data_id)
            mongo_data._workspace_id = workspace_id
            mongo_data.save()
    except Exception as exception:
        logger.error(
            f"ERROR : An error occurred while updating data workspace : {str(exception)}"
        )


@shared_task
def delete_mongo_data(data_id):
    """Delete a data in MongoDB"""
    try:
        try:
            from core_main_app.components.mongo.models import MongoData

            mongo_data = MongoData.objects.get(pk=data_id)
            mongo_data.delete()
        except Exception as exception:
            logger.error(
                f"ERROR : An error occurred while deleting data : {str(exception)}"
            )
    except Exception as exception:
        logger.error(
            f"ERROR : An error occurred while deleting data : {str(exception)}"
        )



""" Get attribute
"""

from django import template

register = template.Library()


@register.filter(name="get_attribute")
def get_attribute(value, arg):
    """get attribute

    https://stackoverflow.com/q/844746

    Args:
        value:
        arg:

    Returns:

    """

    if hasattr(value, str(arg)):
        return getattr(value, arg)
    if isinstance(value, dict) and arg in value:
        return value[arg]
    if arg.isdigit() and len(value) > int(arg):
        return value[int(arg)]
    return ""



"""
    The Database pymongo tool contains the available function relative to database operation (connection)
"""
import logging
import re

logger = logging.getLogger(__name__)


def get_full_text_query(text):
    """Return a full text query.

    Args:
        text: List of keywords

    Returns: The corresponding query

    """
    full_text_query = {}
    word_list = re.sub(r"[^\w]", " ", text, flags=re.UNICODE).split()
    word_list = ['"' + x + '"' for x in word_list]
    word_list = " ".join(word_list)
    if len(word_list) > 0:
        full_text_query = {"$text": {"$search": word_list}}

    return full_text_query


def init_text_index(document_object):
    """Create index for full text search."""
    collection = document_object._get_collection()
    collection.create_index(
        [("$**", "text")], default_language="en", language_override="en"
    )



""" Unit tests of blob templatetag
"""
from unittest.case import TestCase
from unittest.mock import MagicMock

from core_main_app.access_control.exceptions import AccessControlError
from core_main_app.templatetags.blob_tags import blob_metadata, data_blob
from core_main_app.utils.tests_tools.MockUser import create_mock_user


class TestBlobMetadata(TestCase):
    """Test blob metadata"""

    def test_blob_metadata_returns_metadata(self):
        """test_blob_metadata_returns_metadata

        Returns:

        """
        mock_data = MagicMock()
        mock_blob = MagicMock()
        mock_blob.metadata.return_value = [mock_data]
        mock_user = create_mock_user("1")
        result = blob_metadata(mock_blob, mock_user)
        self.assertEqual(result, [mock_data])

    def test_blob_metadata_with_acl_error_returns_none(self):
        """test_blob_metadata_with_acl_error_returns_none

        Returns:

        """
        mock_blob = MagicMock()
        mock_blob.metadata.side_effect = AccessControlError("error")
        mock_user = create_mock_user("1")
        result = blob_metadata(mock_blob, mock_user)
        self.assertIsNone(result)


class TestDataBlob(TestCase):
    """Test data blob"""

    def test_data_blob_returns_blob(self):
        """test_data_blob_returns_blob

        Returns:

        """
        mock_data = MagicMock()
        mock_blob = MagicMock()
        mock_data.blob.return_value = mock_blob
        mock_user = create_mock_user("1")
        result = data_blob(mock_data, mock_user)
        self.assertEqual(result, mock_blob)

    def test_data_blob_with_acl_error_returns_none(self):
        """test_data_blob_with_acl_error_returns_none

        Returns:

        """
        mock_data = MagicMock()
        mock_data.blob.side_effect = AccessControlError("error")
        mock_user = create_mock_user("1")
        result = data_blob(mock_data, mock_user)
        self.assertIsNone(result)



""" Xml operation test class
"""
from collections import OrderedDict
from unittest import TestCase

from core_main_app.commons import exceptions
from core_main_app.utils.xml import (
    raw_xml_to_dict,
    remove_lists_from_xml_dict,
    get_content_by_xpath,
    format_content_xml,
)


class TestRawToDict(TestCase):
    """Test raw_xml_to_dict"""

    def test_raw_to_dict_valid(self):
        """Test valid xml

        Returns:

        """
        # Arrange
        raw_xml = "<root><test>Hello</test></root>"
        expected_dict = OrderedDict(
            [("root", OrderedDict([("test", "Hello")]))]
        )

        # Act
        xml_dict = raw_xml_to_dict(raw_xml)

        # Assert
        self.assertEqual(expected_dict, xml_dict)

    def test_raw_to_dict_throws_exception_when_invalid_xml(self):
        """Test invalid xml

        Returns:

        """
        # Arrange
        raw_xml = "<root><test>Hello</test?</root>"

        # Act # Assert
        with self.assertRaises(exceptions.XMLError):
            raw_xml_to_dict(raw_xml)

    def test_raw_to_dict_without_post_processor(self):
        """Test without post processor

        Returns:

        """
        # Arrange
        raw_xml = "<root><test>Hello</test><test>1</test></root>"
        expected_dict = OrderedDict(
            [("root", OrderedDict([("test", ["Hello", "1"])]))]
        )

        # Act
        xml_dict = raw_xml_to_dict(raw_xml, postprocessor=None)

        # Assert
        self.assertEqual(expected_dict, xml_dict)

    def test_raw_to_dict_with_numeric_post_processor(self):
        """Test with numeric post processor

        Returns:

        """
        # Arrange
        raw_xml = "<root><test>Hello</test><test>1</test></root>"
        expected_dict = OrderedDict(
            [("root", OrderedDict([("test", ["Hello", 1])]))]
        )

        # Act
        xml_dict = raw_xml_to_dict(raw_xml, postprocessor="NUMERIC")

        # Assert
        self.assertEqual(expected_dict, xml_dict)

    def test_raw_to_dict_with_numeric_and_string_post_processor(self):
        """Test with numeric and string post processor

        Returns:

        """
        # Arrange
        raw_xml = "<root><test>Hello</test><test>1</test></root>"
        expected_dict = OrderedDict(
            [("root", OrderedDict([("test", ["Hello", ("1", 1)])]))]
        )

        # Act
        xml_dict = raw_xml_to_dict(raw_xml, postprocessor="NUMERIC_AND_STRING")

        # Assert
        self.assertEqual(expected_dict, xml_dict)

    def test_raw_to_dict_with_callable_post_processor(self):
        """Test with a callable post processor

        Returns:

        """

        def test_processor(path, key, value):
            return key, "test"

        # Arrange
        raw_xml = "<root><test>Hello</test><test>1</test></root>"
        expected_dict = OrderedDict([("root", "test")])

        # Act
        xml_dict = raw_xml_to_dict(raw_xml, postprocessor=test_processor)

        # Assert
        self.assertEqual(expected_dict, xml_dict)

    def test_raw_to_dict_with_unknown_string_raises_error(self):
        """Test with unknown post processor name

        Returns:

        """
        # Arrange
        raw_xml = "<root><test>Hello</test?</root>"

        # Act # Assert
        with self.assertRaises(exceptions.CoreError):
            raw_xml_to_dict(raw_xml, postprocessor="bad_processor")

    def test_raw_to_dict_with_bad_type_raises_error(self):
        """Test with invalid processor type

        Returns:

        """
        # Arrange
        raw_xml = "<root><test>Hello</test?</root>"

        # Act # Assert
        with self.assertRaises(exceptions.CoreError):
            raw_xml_to_dict(raw_xml, postprocessor=1)


class TestRemoveListsFromXmlDict(TestCase):
    """Test remove_lists_from_xml_dict"""

    def test_remove_lists_from_xml_dict_empty_dict_does_nothing(self):
        """Test remove lists from xml dict empty dict does nothing

        Returns:

        """
        # Arrange
        xml_dict = {}
        # Act
        remove_lists_from_xml_dict(xml_dict)
        # Assert
        self.assertEqual(xml_dict, {})

    def test_remove_lists_from_xml_dict_root_list_is_removed(self):
        """Test remove lists from xml dict root list is removed

        Returns:

        """
        # Arrange
        xml_dict = {"list": [{"value": 1}, {"value": 2}]}
        # Act
        remove_lists_from_xml_dict(xml_dict)
        # Assert
        self.assertEqual(xml_dict, {})

    def test_remove_lists_from_xml_dict_root_list_is_not_removed_if_smaller_than_max_size(
        self,
    ):
        """Test remove lists from xml dict root list is not removed if smaller than max size

        Returns:

        """
        # Arrange
        xml_dict = {"list": [{"value": 1}, {"value": 2}]}
        # Act
        remove_lists_from_xml_dict(xml_dict, 3)
        # Assert
        self.assertTrue(xml_dict == {"list": [{"value": 1}, {"value": 2}]})

    def test_remove_lists_from_xml_dict_sub_element_list_is_removed(self):
        """Test remove lists from xml dict sub element list is removed

        Returns:

        """
        # Arrange
        xml_dict = {"root": {"list": [{"value": 1}, {"value": 2}]}}
        # Act
        remove_lists_from_xml_dict(xml_dict)
        # Assert
        self.assertTrue(xml_dict == {"root": {}})

    def test_remove_lists_from_xml_dict_sub_element_list_is_not_removed_if_smaller_than_max_size(
        self,
    ):
        """Test remove lists from xml dict sub element list is not removed if smaller than max size

        Returns:

        """
        # Arrange
        xml_dict = {"root": {"list": [{"value": 1}, {"value": 2}]}}
        # Act
        remove_lists_from_xml_dict(xml_dict, 3)
        # Assert
        self.assertTrue(
            xml_dict == {"root": {"list": [{"value": 1}, {"value": 2}]}}
        )

    def test_remove_lists_from_xml_dict_sub_elements_only_list_is_removed(
        self,
    ):
        """Test remove lists from xml dict sub elements only list is removed

        Returns:

        """
        # Arrange
        xml_dict = {
            "root": {
                "list": [{"value": 1}, {"value": 2}],
                "int": 3,
                "str": "test",
                "dict": {"value": "test"},
            }
        }
        # Act
        remove_lists_from_xml_dict(xml_dict)
        # Assert
        self.assertTrue(
            xml_dict
            == {"root": {"int": 3, "str": "test", "dict": {"value": "test"}}}
        )


class TestGetContentByXpath(TestCase):
    """Test get_content_by_xpath"""

    def test_get_content_and_path_exists(self):
        """test get content and path exists

        Returns:

        """
        # Arrange
        raw_xml = "<root><element>Hello</element></root>"
        # Act
        content = get_content_by_xpath(raw_xml, "/root/element")
        # Assert
        self.assertEqual(content, ["<element>Hello</element>"])

    def test_get_content_path_does_not_exist(self):
        """test get content path does not exist

        Returns:

        """
        # Arrange
        raw_xml = "<root><element>Hello</element></root>"
        # Act
        content = get_content_by_xpath(raw_xml, "/root/test")
        # Assert
        self.assertEqual(content, [])

    def test_get_content_path_found_more_than_once(self):
        """test get content path found more than once

        Returns:

        """
        # Arrange
        raw_xml = (
            "<root><element>Hello</element><element>World</element></root>"
        )
        # Act
        content = get_content_by_xpath(raw_xml, "/root/element")
        # Assert
        self.assertEqual(
            content, ["<element>Hello</element>", "<element>World</element>"]
        )

    def test_get_content_path_is_root(self):
        """test get content path is root

        Returns:

        """
        # Arrange
        raw_xml = "<root><element>Hello</element></root>"
        # Act
        content = get_content_by_xpath(raw_xml, "/root")
        # Assert
        self.assertEqual(content, [raw_xml])

    def test_get_content_path_with_namespace(self):
        """test get content path with namespace

        Returns:

        """
        # Arrange
        raw_xml = """<root xmlns:h="http://www.w3.org/TR/html4/">
        <h:table>
          <h:tr>
            <h:td>Apples</h:td>
            <h:td>Bananas</h:td>
          </h:tr>
        </h:table>
        </root>"""
        # Act
        content = get_content_by_xpath(
            raw_xml,
            "/root/h:table/h:tr/h:td",
            namespaces={"h": "http://www.w3.org/TR/html4/"},
        )
        # Assert
        self.assertEqual(
            content,
            [
                '<h:td xmlns:h="http://www.w3.org/TR/html4/">Apples</h:td>',
                '<h:td xmlns:h="http://www.w3.org/TR/html4/">Bananas</h:td>',
            ],
        )

    def test_get_attribute_path_with_namespace(self):
        """test get attribute path with namespace

        Returns:

        """
        # Arrange
        raw_xml = """<root xmlns:h="http://www.w3.org/TR/html4/">
        <h:table>
          <h:tr class="test">
            <h:td>Apples</h:td>
            <h:td>Bananas</h:td>
          </h:tr>
        </h:table>
        </root>"""
        # Act
        content = get_content_by_xpath(
            raw_xml,
            "/root/h:table/h:tr/@class",
            namespaces={"h": "http://www.w3.org/TR/html4/"},
        )
        # Assert
        self.assertEqual(
            content,
            ["test"],
        )

    def test_get_list_attribute_path_with_namespace(self):
        """test get list attribute path with namespace

        Returns:

        """
        # Arrange
        raw_xml = """<root xmlns:h="http://www.w3.org/TR/html4/">
        <h:table>
          <h:tr>
            <h:td class="class1">Apples</h:td>
            <h:td class="class2">Bananas</h:td>
          </h:tr>
        </h:table>
        </root>"""
        # Act
        content = get_content_by_xpath(
            raw_xml,
            "/root/h:table/h:tr/h:td/@class",
            namespaces={"h": "http://www.w3.org/TR/html4/"},
        )
        # Assert
        self.assertEqual(
            content,
            ["class1", "class2"],
        )

    def test_get_content_returns_list(self):
        """test get content returns list

        Returns:

        """
        # Arrange
        raw_xml = """<root xmlns:h="http://www.w3.org/TR/html4/">
        <h:table>
          <h:tr>
            <h:td>Apples</h:td>
            <h:td>Bananas</h:td>
          </h:tr>
        </h:table>
        </root>"""
        # Act
        content = get_content_by_xpath(
            raw_xml,
            "/root/h:table/h:tr/h:td",
            namespaces={"h": "http://www.w3.org/TR/html4/"},
        )
        # Assert
        self.assertEqual(
            content,
            [
                '<h:td xmlns:h="http://www.w3.org/TR/html4/">Apples</h:td>',
                '<h:td xmlns:h="http://www.w3.org/TR/html4/">Bananas</h:td>',
            ],
        )


class TestFormatContentXml(TestCase):
    """Test Format Content Xml"""

    def test_format_valid_content_xml_returns_content_formatted(self):
        """test_format_valid_content_xml_returns_content_formatted"""

        # Arrange
        xml_string = "<root><test>Hello</test><test>1</test></root>"
        expected_result = (
            "<root>\n  <test>Hello</test>\n  <test>1</test>\n</root>\n"
        )

        # Act
        content = format_content_xml(xml_string)

        # Assert
        self.assertEqual(content, expected_result)

    def test_format_invalid_content_xml_raises_error(self):
        """test_format_invalid_content_xml_raises_error"""

        # Arrange
        xml_string = "<root><test>Hello</test?</root>"

        # Assert
        with self.assertRaises(exceptions.XMLError):
            format_content_xml(xml_string)



""" Data model
"""
from django.conf import settings
from django.contrib.auth.models import User
from django.contrib.postgres.indexes import GinIndex
from django.core.exceptions import ObjectDoesNotExist
from django.db import models
from django.db.models import Q

from core_main_app.access_control.decorators import access_control
from core_main_app.commons import exceptions
from core_main_app.components.abstract_data.models import AbstractData
from core_main_app.components.data.access_control import can_read_blob
from core_main_app.components.template.models import Template
from core_main_app.components.workspace.models import Workspace
from core_main_app.components.blob.models import Blob


# TODO: Create publication workflow manager
# TODO: execute_query / execute_query_full_result -> use find method (RETURN FULL OBJECT)


class Data(AbstractData):
    """Data object"""

    template = models.ForeignKey(
        Template, blank=False, on_delete=models.CASCADE
    )
    user_id = models.CharField(blank=False, max_length=200)
    workspace = models.ForeignKey(
        Workspace, blank=True, on_delete=models.SET_NULL, null=True
    )
    _blob = models.ForeignKey(
        Blob,
        on_delete=models.SET_NULL,
        null=True,
        blank=True,
        default=None,
        related_name="_metadata",
    )

    class Meta:
        """Meta"""

        verbose_name = "Data"
        verbose_name_plural = "Data"
        indexes = [
            models.Index(
                fields=[
                    "title",
                    "last_modification_date",
                    "template",
                    "user_id",
                ]
            ),
            GinIndex(fields=["vector_column"]),
        ]

    @property
    def owner_name(self):
        """Get owner name

        Returns:

        """
        return User.objects.get(pk=self.user_id).username

    @access_control(can_read_blob)
    def blob(self, user):
        """

        Args:
            user:

        Returns:

        """
        return self._blob

    def get_dict_content(self):
        """Get dict_content from object or from MongoDB

        Returns:

        """
        if settings.MONGODB_INDEXING:
            from core_main_app.components.mongo.models import MongoData

            return MongoData.objects.get(pk=self.id).dict_content
        return self.dict_content

    @staticmethod
    def get_all(order_by_field):
        """Get all data.

        Args:
            order_by_field: Order by field.

        Returns:

        """
        return Data.objects.all().order_by(
            *[field.replace("+", "") for field in order_by_field]
        )

    @staticmethod
    def get_all_except(order_by_field, id_list=None):
        """Get all data except for the ones with ID within the provided list.

        Args:
            id_list:
            order_by_field:

        Returns:
        """
        if id_list is None:
            return Data.get_all(
                [field.replace("+", "") for field in order_by_field]
            )

        return Data.objects.exclude(pk__in=id_list).order_by(
            *[field.replace("+", "") for field in order_by_field]
        )

    @staticmethod
    def get_all_by_user_id(user_id, order_by_field):
        """Get all data relative to the given user id

        Args:
            user_id:
            order_by_field: Order by field.

        Returns:

        """
        return Data.objects.filter(user_id=str(user_id)).order_by(
            *[field.replace("+", "") for field in order_by_field]
        )

    @staticmethod
    def get_all_except_user_id(user_id, order_by_field):
        """Get all data non relative to the given user id

        Args:
            user_id:
            order_by_field:

        Returns:

        """
        return Data.objects.exclude(user_id__in=str(user_id)).order_by(
            *[field.replace("+", "") for field in order_by_field]
        )

    @staticmethod
    def get_all_by_id_list(list_id, order_by_field):
        """Return the object with the given list id.

        Args:
            list_id:
            order_by_field:

        Returns:
            Object collection
        """
        return Data.objects.filter(pk__in=list_id).order_by(
            *[field.replace("+", "") for field in order_by_field]
        )

    @staticmethod
    def get_by_id(data_id):
        """Return the object with the given id.

        Args:
            data_id:

        Returns:
            Data (obj): Data object with the given id

        """
        try:
            return Data.objects.get(pk=data_id)
        except ObjectDoesNotExist as e:
            raise exceptions.DoesNotExist(str(e))
        except Exception as ex:
            raise exceptions.ModelError(str(ex))

    @staticmethod
    def execute_query(query, order_by_field):
        """Execute a query.

        Args:
            query:
            order_by_field: Order by field.

        Returns:

        """
        return (
            Data.objects.filter(query)
            .order_by(*[field.replace("+", "") for field in order_by_field])
            .all()
        )

    @staticmethod
    def get_all_by_workspace(workspace, order_by_field):
        """Get all data that belong to the workspace.

        Args:
            workspace:
            order_by_field:

        Returns:

        """

        if workspace is None:
            workspace_q = Q(workspace__isnull=True)
        else:
            workspace_q = Q(workspace=workspace)
        return Data.objects.filter(workspace_q).order_by(
            *[field.replace("+", "") for field in order_by_field]
        )

    @staticmethod
    def get_all_by_list_template(list_template, order_by_field):
        """Get all data that belong to the list of template.

        Args:
            list_template:
            order_by_field:

        Returns:

        """
        return Data.objects.filter(template__in=list_template).order_by(
            *[field.replace("+", "") for field in order_by_field]
        )

    @staticmethod
    def get_none():
        """Return None object, used by data.

        Returns:

        """
        return Data.objects.none()

    def __str__(self):
        """Return Data object as string

        Returns:

        """
        return self.title



""" Test units for User Preferences
"""

from unittest.case import TestCase
from unittest.mock import patch

from django.core.exceptions import ValidationError, ObjectDoesNotExist

from core_main_app.commons import exceptions
from core_main_app.components.user_preferences import (
    api as user_preferences_api,
)
from core_main_app.components.user_preferences.admin_site import (
    CustomUserPreferencesAdmin,
)
from core_main_app.components.user_preferences.models import UserPreferences
from core_main_app.utils.tests_tools.MockUser import create_mock_user


class TestUserPreferencesUpsert(TestCase):
    """Test User Preferences Upsert"""

    @patch.object(UserPreferences, "save")
    def test_user_preferences_upsert_returns_object(self, mock_save):
        """test_user_preferences_upsert_return_object

        Args:
            mock_save:

        Returns:

        """
        # Arrange
        mock_user_preferences = UserPreferences(
            user_id="1", timezone="Europe/Paris"
        )

        mock_save.return_value = mock_user_preferences

        # Act
        result = user_preferences_api.upsert(
            mock_user_preferences, create_mock_user("1")
        )

        # Assert
        self.assertIsInstance(result, UserPreferences)

    @patch.object(UserPreferences, "save")
    def test_user_preferences_upsert_raise_validation_error_when_invalid_timezone_set(
        self, mock_save
    ):
        """test_user_preferences_upsert_raise_validation_error_when_invalid_timezone_set

        Args:
            mock_save:

        Returns:

        """
        # Arrange
        mock_user_preferences = UserPreferences(
            user_id="1", timezone="bad timezone"
        )

        mock_save.side_effect = exceptions.ModelError("")

        # Act + Assert
        with self.assertRaises(ValidationError):
            user_preferences_api.upsert(
                mock_user_preferences, create_mock_user("1")
            )

    @patch.object(UserPreferences, "save")
    def test_user_preferences_upsert_raise_error_when_user_has_already_user_preferences(
        self, mock_save
    ):
        """test_user_preferences_upsert_raise_error_when_user_has_already_user_preferences

        Args:
            mock_save:

        Returns:

        """
        # Arrange
        mock_user_preferences = UserPreferences(
            user_id="1", timezone="Europe/Paris"
        )

        mock_save.side_effect = exceptions.ModelError("")

        # Act + Assert
        with self.assertRaises(exceptions.ModelError):
            user_preferences_api.upsert(
                mock_user_preferences, create_mock_user("1")
            )


class TestUserPreferencesDelete(TestCase):
    """Test User Preferences Delete"""

    @patch.object(UserPreferences, "delete")
    def test_delete_user_preferences_deletes(self, mock_delete):
        """test_delete_user_preferences_deletes

        Args:
            mock_delete:

        Returns:

        """
        # Arrange
        mock_user_preferences = UserPreferences(user_id="1", timezone="test")

        mock_delete.return_value = []

        # Act
        user_preferences_api.delete(
            mock_user_preferences, create_mock_user("1")
        )

    @patch.object(UserPreferences, "delete")
    def test_delete_user_preferences_raises_dne_exception(self, mock_delete):
        """test_delete_user_preferences_raises_dne_exception

        Args:
            mock_delete:
        Returns:

        """
        # Arrange
        mock_user_preferences = UserPreferences(user_id="1", timezone="test")
        mock_delete.side_effect = exceptions.DoesNotExist("")
        # Act + Assert
        with self.assertRaises(exceptions.DoesNotExist):
            user_preferences_api.delete(
                mock_user_preferences, create_mock_user("1")
            )


class TestUserPreferencesGetByUser(TestCase):
    """Test User Preferences Get By User"""

    @patch.object(UserPreferences, "objects")
    def test_get_user_preferences_by_user_returns_object(self, mock_objects):
        """test_get_user_preferences_by_user_return_object

        Args:
            mock_objects:

        Returns:

        """
        # Arrange
        mock_user_preferences = UserPreferences(user_id="1", timezone="test")

        mock_objects.get.return_value = mock_user_preferences

        # Act
        result = user_preferences_api.get_by_user(
            create_mock_user("1", is_superuser=True)
        )

        # Assert
        self.assertEquals(result.timezone, "test")
        self.assertEquals(result.user_id, "1")

    @patch.object(UserPreferences, "objects")
    def test_get_user_preferences_by_user_raises_dne_exception(
        self, mock_objects
    ):
        """test_get_user_preferences_by_user_raises_dne_exception

        Args:
            mock_objects

        Returns:

        """
        # Arrange
        mock_objects.get.side_effect = ObjectDoesNotExist("")

        # Act + Assert
        with self.assertRaises(exceptions.DoesNotExist):
            user_preferences_api.get_by_user(
                create_mock_user("1", is_superuser=True)
            )

    @patch.object(UserPreferences, "objects")
    def test_get_user_preferences_by_user_raises_model_error_exception(
        self, mock_objects
    ):
        """test_get_user_preferences_by_user_raises_model_error_exception

        Args:

        Returns:

        """
        # Arrange
        mock_objects.get.side_effect = exceptions.ModelError("")

        # Act + Assert
        with self.assertRaises(exceptions.ModelError):
            user_preferences_api.get_by_user(
                create_mock_user("1", is_superuser=True)
            )


class TestCustomUserPreferencesAdmin(TestCase):
    """Test Custom User Preferences Admin"""

    def test_custom_user_preferences_admin(self):
        """test_custom_user_preferences_admin

        Args:

        Returns:

        """
        # Arrange
        custom_admin_site = CustomUserPreferencesAdmin(UserPreferences, None)

        # Assert
        self.assertTrue("user_id" in custom_admin_site.list_display)
        self.assertTrue("timezone" in custom_admin_site.list_display)

    def test_custom_user_preferences_admin_add_permission(self):
        """test_custom_user_preferences_admin_add_permission

        Args:

        Returns:

        """
        # Arrange
        custom_admin_site = CustomUserPreferencesAdmin(UserPreferences, None)

        # Assert
        self.assertFalse(custom_admin_site.has_add_permission(request=None))



""" MongoData API
"""
from core_main_app.access_control.decorators import access_control
from core_main_app.components.data import (
    access_control as data_api_access_control,
)
from core_main_app.settings import DATA_SORTING_FIELDS


@access_control(data_api_access_control.can_read_data_query)
def execute_mongo_query(
    json_query,
    user,
    workspace_filter=None,
    user_filter=None,
    order_by_field=DATA_SORTING_FIELDS,
):
    """

    Args:
        json_query:
        user:
        workspace_filter:
        user_filter:
        order_by_field:

    Returns:

    """
    from core_main_app.components.mongo.models import MongoData

    return MongoData.execute_query(json_query, order_by_field)


@access_control(data_api_access_control.can_read_aggregate_query)
def aggregate(pipeline, user):
    """Execute an aggregate on the Data collection.

    Args:
        pipeline:
        user:

    Returns:

    """
    from core_main_app.components.mongo.models import MongoData

    return MongoData.aggregate(pipeline)



"""
Version Manager model
"""
from abc import abstractmethod

from django.core.validators import RegexValidator
from django.db import models, IntegrityError

from core_main_app.commons import exceptions
from core_main_app.commons.regex import NOT_EMPTY_OR_WHITESPACES
from core_main_app.utils.validation.regex_validation import (
    not_empty_or_whitespaces,
)


# TODO: could make is_disabled a Status with other possible values taken from an enum


class Version(models.Model):
    """Version"""

    is_current = models.BooleanField(default=False)
    is_disabled = models.BooleanField(default=False)

    class Meta:
        """Meta"""

        abstract = True


class VersionManager(models.Model):
    """Version Manager"""

    title = models.CharField(
        unique=True,
        validators=[
            RegexValidator(
                regex=NOT_EMPTY_OR_WHITESPACES,
                message="Title must not be empty or only whitespaces",
                code="invalid_title",
            ),
        ],
        max_length=200,
    )
    user = models.CharField(
        blank=True, max_length=200, null=True, default=None
    )
    is_disabled = models.BooleanField(default=False)
    # TODO: for now, set _cls for backward compatibility in apis
    _cls = models.CharField(default="VersionManager", max_length=200)
    creation_date = models.DateTimeField(auto_now_add=True)

    class Meta:
        """Meta"""

        abstract = True

    @property
    @abstractmethod
    def version_set(self):
        """version_set

        Returns:

        """
        raise NotImplementedError("Implement in the child class.")

    @property
    @abstractmethod
    def class_name(self):
        """class_name

        Returns:

        """
        raise NotImplementedError("Implement in the child class.")

    @property
    def current_version(self):
        """current_version"""
        return self.version_set.get(is_current=True)

    @property
    def disabled_version_set(self):
        """disabled_version_set"""
        return self.version_set.filter(is_disabled=True)

    @property
    def versions(self):
        """versions (backward compatibility)"""
        return [str(version.id) for version in self.version_set]

    @property
    def current(self):
        """current (backward compatibility)"""
        return str(self.current_version.id)

    @property
    def disabled_versions(self):
        """disabled_versions (backward compatibility)"""
        return [str(version.id) for version in self.disabled_version_set]

    def save_version_manager(self):
        """Custom save.

        Returns:
            Saved Instance.

        """
        try:
            self._cls = self.class_name
            self.clean()
            self.save()
        except IntegrityError as exception:
            raise exceptions.NotUniqueError(str(exception))
        except Exception as ex:
            raise exceptions.ModelError(str(ex))

    def clean(self):
        """Clean is called before saving

        Returns:

        """
        not_empty_or_whitespaces(self.title)
        self.title = self.title.strip()

    def __str__(self):
        """Version Manager as string

        Returns:

        """
        return self.title



""" TemplateXslRendering model
"""
from django.core.exceptions import ObjectDoesNotExist
from django.db import models

from core_main_app.commons import exceptions
from core_main_app.components.template.models import Template
from core_main_app.components.xsl_transformation.models import (
    XslTransformation,
)


class TemplateXslRendering(models.Model):
    """TemplateXslRendering object"""

    template = models.OneToOneField(
        Template, on_delete=models.CASCADE, blank=False
    )
    list_xslt = models.ForeignKey(
        XslTransformation,
        on_delete=models.SET_NULL,
        blank=True,
        related_name="list_xslt",
        null=True,
    )
    default_detail_xslt = models.ForeignKey(
        XslTransformation,
        on_delete=models.SET_NULL,
        blank=True,
        related_name="default_detail_xslt",
        null=True,
    )
    list_detail_xslt = models.ManyToManyField(
        XslTransformation,
        blank=True,
        default=[],
    )

    @staticmethod
    def get_by_id(template_xsl_rendering_id):
        """Get a TemplateXslRendering document by its id.

        Args:
            template_xsl_rendering_id: Id.

        Returns:
            TemplateXslRendering object.

        Raises:
            DoesNotExist: The TemplateXslRendering doesn't exist.
            ModelError: Internal error during the process.

        """
        try:
            return TemplateXslRendering.objects.get(
                pk=template_xsl_rendering_id
            )
        except ObjectDoesNotExist as exception:
            raise exceptions.DoesNotExist(str(exception))
        except Exception as ex:
            raise exceptions.ModelError(str(ex))

    @staticmethod
    def get_by_template_id(template_id):
        """Get TemplateXslRendering by its template id.

        Args:
            template_id: Template id.

        Returns:
            The TemplateXslRendering instance.

        Raises:
            DoesNotExist: The TemplateXslRendering doesn't exist.
            ModelError: Internal error during the process.

        """
        try:
            return TemplateXslRendering.objects.get(template=template_id)
        except ObjectDoesNotExist as exception:
            raise exceptions.DoesNotExist(str(exception))
        except Exception as exception:
            raise exceptions.ModelError(str(exception))

    @staticmethod
    def get_by_template_hash(template_hash):
        """Get TemplateXslRendering by its template hash.

        Args:
            template_hash: Template hash.

        Returns:
            The TemplateXslRendering instance.

        Raises:
            DoesNotExist: The TemplateXslRendering doesn't exist.
            ModelError: Internal error during the process.

        """
        try:
            return TemplateXslRendering.objects.get(
                template__hash=template_hash
            )
        except ObjectDoesNotExist as exception:
            raise exceptions.DoesNotExist(str(exception))
        except Exception as exception:
            raise exceptions.ModelError(str(exception))

    @staticmethod
    def get_all():
        """Get all TemplateXslRendering.

        Returns:
            List of TemplateXslRendering.

        """
        return TemplateXslRendering.objects.all()

    def __str__(self):
        """Template Xsl Rendering as string

        Returns:

        """
        return str(self.template)
